"""
Tests
"""

import pytest

from ArWikiCats.new_resolvers.sports_resolvers.match_labs import find_teams_2025

test_new = {
    "Women's sports": "رياضات نسائية",
    "Women's sports seasons": "مواسم رياضات نسائية",
    "sports seasons": "مواسم رياضات",
}


@pytest.mark.parametrize("category, expected_key", test_new.items(), ids=test_new.keys())
@pytest.mark.fast
def test_new_data(category: str, expected_key: str) -> None:
    label = find_teams_2025(category)
    assert label == ""  # expected_key


data00 = {
    "football": "كرة قدم",
    "sports competitions": "منافسات رياضية",
    "women's sports clubs": "أندية رياضية نسائية",
    "women's sports competitions": "منافسات رياضية نسائية",
    "women's sports leagues": "دوريات رياضية نسائية",
    "women's sports organizations": "منظمات رياضية نسائية",
    "sports films": "أفلام رياضية",
    "women's sports teams": "فرق رياضية نسائية",
}

data6 = {
    "Category:amateur football competitions": "منافسات كرة قدم للهواة",
    "amateur football competitions": "منافسات كرة قدم للهواة",
    "american football teams": "فرق كرة قدم أمريكية",
    "american football venues": "ملاعب كرة قدم أمريكية",
    "athletics competitions": "منافسات ألعاب قوى",
    "athletics navigational boxes": "صناديق تصفح ألعاب قوى",
    "athletics venues": "ملاعب ألعاب قوى",
    "australian rules football clubs": "أندية كرة قدم أسترالية",
    "badminton venues": "ملاعب تنس ريشة",
    "baseball playerss": "لاعبو كرة قاعدة",
    "baseball venues": "ملاعب كرة قاعدة",
    "basketball competitions": "منافسات كرة سلة",
    "basketball leagues": "دوريات كرة سلة",
    "basketball people": "أعلام كرة سلة",
    "basketball playerss": "لاعبو كرة سلة",
    "basketball teams": "فرق كرة سلة",
    "boxing competitions": "منافسات بوكسينغ",
    "canadian football teams": "فرق كرة قدم كندية",
    "canoeing and kayaking competitions": "منافسات كانو-كاياك",
    "cricket clubs": "أندية كريكت",
    "cycling clubs": "أندية سباق دراجات هوائية",
    "cycling events": "أحداث سباق دراجات هوائية",
    "field hockey clubs": "أندية هوكي ميدان",
    "field hockey people": "أعلام هوكي ميدان",
    "field hockey playerss": "لاعبو هوكي ميدان",
    "figure skating films": "أفلام تزلج فني",
    "figure skating people": "أعلام تزلج فني",
    "football people": "أعلام كرة قدم",
    "football venues": "ملاعب كرة قدم",
    "gaelic football clubs": "أندية كرة قدم غالية",
    "gaelic football competitions": "منافسات كرة قدم غالية",
    "gymnastics competitions": "منافسات جمباز",
    "gymnastics organizations": "منظمات جمباز",
    "handball competitions": "منافسات كرة يد",
    "ice hockey leagues": "دوريات هوكي جليد",
    "ice hockey people": "أعلام هوكي جليد",
    "ice hockey playerss": "لاعبو هوكي جليد",
    "ice hockey venues": "ملاعب هوكي جليد",
    "judo competitions": "منافسات جودو",
    "men's football competitions": "منافسات كرة قدم رجالية",
    "men's football leagues": "دوريات كرة قدم رجالية",
    "motorsport competitions": "منافسات رياضة محركات",
    "motorsport venues": "ملاعب رياضة محركات",
    "rowing clubs": "أندية تجديف",
    "rugby league playerss": "لاعبو دوري رجبي",
    "rugby league venues": "ملاعب دوري رجبي",
    "rugby union competitions": "منافسات اتحاد رجبي",
    "rugby union teams": "فرق اتحاد رجبي",
    "rugby union tournaments": "بطولات اتحاد رجبي",
    "shooting competitions": "منافسات رماية",
    "soccer matches": "مباريات كرة قدم",
    "soccer venues": "ملاعب كرة قدم",
    "softball venues": "ملاعب كرة لينة",
    "table tennis clubs": "أندية كرة طاولة",
    "tennis people": "أعلام كرة مضرب",
    "tennis tournaments": "بطولات كرة مضرب",
    "track and field competitions": "منافسات سباقات مضمار وميدان",
    "volleyball clubs": "أندية كرة طائرة",
    "volleyball competitions": "منافسات كرة طائرة",
    "wheelchair basketball competitions": "منافسات كرة سلة على كراسي متحركة",
    "wheelchair basketball leagues": "دوريات كرة سلة على كراسي متحركة",
    "wheelchair basketball playerss": "لاعبو كرة سلة على كراسي متحركة",
    "wheelchair basketball teams": "فرق كرة سلة على كراسي متحركة",
    "wheelchair basketball templates": "قوالب كرة سلة على كراسي متحركة",
    "wheelchair basketball venues": "ملاعب كرة سلة على كراسي متحركة",
    "wheelchair handball competitions": "منافسات كرة يد على كراسي متحركة",
    "wheelchair rugby competitions": "منافسات رجبي على كراسي متحركة",
    "wheelchair rugby people": "أعلام رجبي على كراسي متحركة",
    "wheelchair rugby templates": "قوالب رجبي على كراسي متحركة",
    "wheelchair tennis tournaments": "بطولات كرة مضرب على كراسي متحركة",
    "women's basketball competitions": "منافسات كرة سلة نسائية",
    "women's basketball leagues": "دوريات كرة سلة نسائية",
    "women's cricket teams": "فرق كريكت نسائية",
    "women's football clubs": "أندية كرة قدم نسائية",
    "women's soccer leagues": "دوريات كرة قدم نسائية",
    "youth athletics competitions": "منافسات ألعاب قوى شبابية",
    "youth football competitions": "منافسات كرة قدم شبابية",
}

teams_2025_data_womens = {
    "womens youth football": "كرة قدم للشابات",
    "womens youth football centers": "لاعبات وسط كرة قدم للشابات",
    "womens youth football central defenders": "مدافعات مركزيات كرة قدم للشابات",
    "womens youth football centres": "لاعبات وسط كرة قدم للشابات",
    "womens youth football champions": "أبطال كرة قدم للشابات",
    "womens youth football clubs": "أندية كرة قدم للشابات",
    "womens youth football coaches": "مدربو كرة قدم للشابات",
    "womens youth football competitions": "منافسات كرة قدم للشابات",
    "womens youth football defencemen": "مدافعات كرة قدم للشابات",
    "womens youth football defenders": "مدافعات كرة قدم للشابات",
    "womens youth football defensive backs": "مدافعات خلفيات كرة قدم للشابات",
    "womens youth football defensive linemen": "مدافعات خط كرة قدم للشابات",
    "womens youth football directors": "مدراء كرة قدم للشابات",
    "womens youth football drop kickers": "مسددات ركلات كرة قدم للشابات",
    "womens youth football events": "أحداث كرة قدم للشابات",
    "womens youth football films": "أفلام كرة قدم للشابات",
    "womens youth football finals": "نهائيات كرة قدم للشابات",
    "womens youth football forwards": "مهاجمات كرة قدم للشابات",
    "womens youth football fullbacks": "مدافعات كرة قدم للشابات",
    "womens youth football goalkeepers": "حارسات مرمى كرة قدم للشابات",
    "womens youth football goaltenders": "حارسات مرمى كرة قدم للشابات",
    "womens youth football guards": "حارسات كرة قدم للشابات",
    "womens youth football halfbacks": "ظهيرات مساعدات كرة قدم للشابات",
    "womens youth football home stadiums": "ملاعب كرة قدم للشابات",
    "womens youth football inside forwards": "مهاجمات داخليات كرة قدم للشابات",
    "womens youth football journalists": "صحفيات كرة قدم للشابات",
    "womens youth football kickers": "راكلات كرة قدم للشابات",
    "womens youth football leagues": "دوريات كرة قدم للشابات",
    "womens youth football left wingers": "جناحات يسار كرة قدم للشابات",
    "womens youth football linebackers": "ظهيرات كرة قدم للشابات",
    "womens youth football lists": "قوائم كرة قدم للشابات",
    "womens youth football manager history": "تاريخ مدربو كرة قدم للشابات",
    "womens youth football managers": "مدربو كرة قدم للشابات",
    "womens youth football matches": "مباريات كرة قدم للشابات",
    "womens youth football midfielders": "لاعبات وسط كرة قدم للشابات",
    "womens youth football navigational boxes": "صناديق تصفح كرة قدم للشابات",
    "womens youth football non-profit organizations": "منظمات غير ربحية كرة قدم للشابات",
    "womens youth football non-profit publishers": "ناشرون غير ربحيون كرة قدم للشابات",
    "womens youth football offensive linemen": "مهاجمات خط كرة قدم للشابات",
    "womens youth football organisations": "منظمات كرة قدم للشابات",
    "womens youth football organizations": "منظمات كرة قدم للشابات",
    "womens youth football outside forwards": "مهاجمات خارجيات كرة قدم للشابات",
    "womens youth football peoplee": "أعلام كرة قدم للشابات",
    "womens youth football placekickers": "مسددات كرة قدم للشابات",
    "womens youth football players": "لاعبات كرة قدم للشابات",
    "womens youth football playerss": "لاعبات كرة قدم للشابات",
    "womens youth football point guards": "لاعبات هجوم خلفي كرة قدم للشابات",
    "womens youth football positions": "مراكز كرة قدم للشابات",
    "womens youth football power forwards": "مهاجمات قويات الجسم كرة قدم للشابات",
    "womens youth football quarterbacks": "ظهيرات رباعيات كرة قدم للشابات",
    "womens youth football receivers": "مستقبلات كرة قدم للشابات",
    "womens youth football records": "سجلات كرة قدم للشابات",
    "womens youth football records and statistics": "سجلات وإحصائيات كرة قدم للشابات",
    "womens youth football referees": "حكام كرة قدم للشابات",
    "womens youth football results": "نتائج كرة قدم للشابات",
    "womens youth football right wingers": "جناحات يمين كرة قدم للشابات",
    "womens youth football rivalries": "دربيات كرة قدم للشابات",
    "womens youth football running backs": "راكضات للخلف كرة قدم للشابات",
    "womens youth football scouts": "كشافة كرة قدم للشابات",
    "womens youth football shooting guards": "مدافعات مسددات كرة قدم للشابات",
    "womens youth football small forwards": "مهاجمات صغيرات الجسم كرة قدم للشابات",
    "womens youth football sports-people": "رياضيات كرة قدم للشابات",
    "womens youth football squads": "تشكيلات كرة قدم للشابات",
    "womens youth football statistics": "إحصائيات كرة قدم للشابات",
    "womens youth football tackles": "مصطدمات كرة قدم للشابات",
    "womens youth football teams": "فرق كرة قدم للشابات",
    "womens youth football templates": "قوالب كرة قدم للشابات",
    "womens youth football tournaments": "بطولات كرة قدم للشابات",
    "womens youth football trainers": "مدربو كرة قدم للشابات",
    "womens youth football umpires": "حكام كرة قدم للشابات",
    "womens youth football utility players": "لاعبات مراكز متعددة كرة قدم للشابات",
    "womens youth football venues": "ملاعب كرة قدم للشابات",
    "womens youth football wide receivers": "مستقبلات واسعات كرة قدم للشابات",
    "womens youth football wing halves": "جناحات نصفيات كرة قدم للشابات",
    "womens youth football wingers": "جناحات كرة قدم للشابات",
    "womens football": "كرة قدم نسائية",
    "womens football centers": "لاعبات وسط كرة قدم نسائية",
    "womens football central defenders": "مدافعات مركزيات كرة قدم نسائية",
    "womens football centres": "لاعبات وسط كرة قدم نسائية",
    "womens football champions": "أبطال كرة قدم نسائية",
    "womens football clubs": "أندية كرة قدم نسائية",
    "womens football coaches": "مدربو كرة قدم نسائية",
    "womens football competitions": "منافسات كرة قدم نسائية",
    "womens football defencemen": "مدافعات كرة قدم نسائية",
    "womens football defenders": "مدافعات كرة قدم نسائية",
    "womens football defensive backs": "مدافعات خلفيات كرة قدم نسائية",
    "womens football defensive linemen": "مدافعات خط كرة قدم نسائية",
    "womens football directors": "مدراء كرة قدم نسائية",
    "womens football drop kickers": "مسددات ركلات كرة قدم نسائية",
    "womens football events": "أحداث كرة قدم نسائية",
    "womens football films": "أفلام كرة قدم نسائية",
    "womens football finals": "نهائيات كرة قدم نسائية",
    "womens football forwards": "مهاجمات كرة قدم نسائية",
    "womens football fullbacks": "مدافعات كرة قدم نسائية",
    "womens football goalkeepers": "حارسات مرمى كرة قدم نسائية",
    "womens football goaltenders": "حارسات مرمى كرة قدم نسائية",
    "womens football guards": "حارسات كرة قدم نسائية",
    "womens football halfbacks": "ظهيرات مساعدات كرة قدم نسائية",
    "womens football home stadiums": "ملاعب كرة قدم نسائية",
    "womens football inside forwards": "مهاجمات داخليات كرة قدم نسائية",
    "womens football journalists": "صحفيات كرة قدم نسائية",
    "womens football kickers": "راكلات كرة قدم نسائية",
    "womens football leagues": "دوريات كرة قدم نسائية",
    "womens football left wingers": "جناحات يسار كرة قدم نسائية",
    "womens football linebackers": "ظهيرات كرة قدم نسائية",
    "womens football lists": "قوائم كرة قدم نسائية",
    "womens football manager history": "تاريخ مدربو كرة قدم نسائية",
    "womens football managers": "مدربو كرة قدم نسائية",
    "womens football matches": "مباريات كرة قدم نسائية",
    "womens football midfielders": "لاعبات وسط كرة قدم نسائية",
    "womens football navigational boxes": "صناديق تصفح كرة قدم نسائية",
    "womens football non-profit organizations": "منظمات غير ربحية كرة قدم نسائية",
    "womens football non-profit publishers": "ناشرون غير ربحيون كرة قدم نسائية",
    "womens football offensive linemen": "مهاجمات خط كرة قدم نسائية",
    "womens football organisations": "منظمات كرة قدم نسائية",
    "womens football organizations": "منظمات كرة قدم نسائية",
    "womens football outside forwards": "مهاجمات خارجيات كرة قدم نسائية",
    "womens football peoplee": "أعلام كرة قدم نسائية",
    "womens football placekickers": "مسددات كرة قدم نسائية",
    "womens football players": "لاعبات كرة قدم نسائية",
    "womens football playerss": "لاعبات كرة قدم نسائية",
    "womens football point guards": "لاعبات هجوم خلفي كرة قدم نسائية",
    "womens football positions": "مراكز كرة قدم نسائية",
    "womens football power forwards": "مهاجمات قويات الجسم كرة قدم نسائية",
    "womens football quarterbacks": "ظهيرات رباعيات كرة قدم نسائية",
    "womens football receivers": "مستقبلات كرة قدم نسائية",
    "womens football records": "سجلات كرة قدم نسائية",
    "womens football records and statistics": "سجلات وإحصائيات كرة قدم نسائية",
    "womens football referees": "حكام كرة قدم نسائية",
    "womens football results": "نتائج كرة قدم نسائية",
    "womens football right wingers": "جناحات يمين كرة قدم نسائية",
    "womens football rivalries": "دربيات كرة قدم نسائية",
    "womens football running backs": "راكضات للخلف كرة قدم نسائية",
    "womens football scouts": "كشافة كرة قدم نسائية",
    "womens football shooting guards": "مدافعات مسددات كرة قدم نسائية",
    "womens football small forwards": "مهاجمات صغيرات الجسم كرة قدم نسائية",
    "womens football sports-people": "رياضيات كرة قدم نسائية",
    "womens football squads": "تشكيلات كرة قدم نسائية",
    "womens football statistics": "إحصائيات كرة قدم نسائية",
    "womens football tackles": "مصطدمات كرة قدم نسائية",
    "womens football teams": "فرق كرة قدم نسائية",
    "womens football templates": "قوالب كرة قدم نسائية",
    "womens football tournaments": "بطولات كرة قدم نسائية",
    "womens football trainers": "مدربو كرة قدم نسائية",
    "womens football umpires": "حكام كرة قدم نسائية",
    "womens football utility players": "لاعبات مراكز متعددة كرة قدم نسائية",
    "womens football venues": "ملاعب كرة قدم نسائية",
    "womens football wide receivers": "مستقبلات واسعات كرة قدم نسائية",
    "womens football wing halves": "جناحات نصفيات كرة قدم نسائية",
    "womens football wingers": "جناحات كرة قدم نسائية",
}

teams_2025_data = {
    "amateur football": "كرة قدم للهواة",
    "amateur football centers": "لاعبو وسط كرة قدم للهواة",
    "amateur football central defenders": "قلوب دفاع كرة قدم للهواة",
    "amateur football centres": "لاعبو وسط كرة قدم للهواة",
    "amateur football champions": "أبطال كرة قدم للهواة",
    "amateur football clubs": "أندية كرة قدم للهواة",
    "amateur football coaches": "مدربو كرة قدم للهواة",
    "amateur football competitions": "منافسات كرة قدم للهواة",
    "amateur football defencemen": "مدافعو كرة قدم للهواة",
    "amateur football defenders": "مدافعو كرة قدم للهواة",
    "amateur football defensive backs": "مدافعون خلفيون كرة قدم للهواة",
    "amateur football defensive linemen": "مدافعو خط كرة قدم للهواة",
    "amateur football directors": "مدراء كرة قدم للهواة",
    "amateur football drop kickers": "مسددو ركلات كرة قدم للهواة",
    "amateur football events": "أحداث كرة قدم للهواة",
    "amateur football films": "أفلام كرة قدم للهواة",
    "amateur football finals": "نهائيات كرة قدم للهواة",
    "amateur football forwards": "مهاجمو كرة قدم للهواة",
    "amateur football fullbacks": "مدافعو كرة قدم للهواة",
    "amateur football goalkeepers": "حراس مرمى كرة قدم للهواة",
    "amateur football goaltenders": "حراس مرمى كرة قدم للهواة",
    "amateur football guards": "حراس كرة قدم للهواة",
    "amateur football halfbacks": "أظهرة مساعدون كرة قدم للهواة",
    "amateur football home stadiums": "ملاعب كرة قدم للهواة",
    "amateur football inside forwards": "مهاجمون داخليون كرة قدم للهواة",
    "amateur football journalists": "صحفيو كرة قدم للهواة",
    "amateur football kickers": "راكلو كرة قدم للهواة",
    "amateur football leagues": "دوريات كرة قدم للهواة",
    "amateur football left wingers": "أجنحة يسار كرة قدم للهواة",
    "amateur football linebackers": "أظهرة كرة قدم للهواة",
    "amateur football lists": "قوائم كرة قدم للهواة",
    "amateur football manager history": "تاريخ مدربو كرة قدم للهواة",
    "amateur football managers": "مدربو كرة قدم للهواة",
    "amateur football matches": "مباريات كرة قدم للهواة",
    "amateur football midfielders": "لاعبو وسط كرة قدم للهواة",
    "amateur football navigational boxes": "صناديق تصفح كرة قدم للهواة",
    "amateur football non-profit organizations": "منظمات غير ربحية كرة قدم للهواة",
    "amateur football non-profit publishers": "ناشرون غير ربحيون كرة قدم للهواة",
    "amateur football offensive linemen": "مهاجمو خط كرة قدم للهواة",
    "amateur football organisations": "منظمات كرة قدم للهواة",
    "amateur football organizations": "منظمات كرة قدم للهواة",
    "amateur football outside forwards": "مهاجمون خارجيون كرة قدم للهواة",
    "amateur football peoplee": "أعلام كرة قدم للهواة",
    "amateur football placekickers": "مسددو كرة قدم للهواة",
    "amateur football players": "لاعبو كرة قدم للهواة",
    "amateur football playerss": "لاعبو كرة قدم للهواة",
    "amateur football point guards": "لاعبو هجوم خلفي كرة قدم للهواة",
    "amateur football positions": "مراكز كرة قدم للهواة",
    "amateur football power forwards": "مهاجمون أقوياء الجسم كرة قدم للهواة",
    "amateur football quarterbacks": "أظهرة رباعيون كرة قدم للهواة",
    "amateur football receivers": "مستقبلو كرة قدم للهواة",
    "amateur football records": "سجلات كرة قدم للهواة",
    "amateur football records and statistics": "سجلات وإحصائيات كرة قدم للهواة",
    "amateur football referees": "حكام كرة قدم للهواة",
    "amateur football results": "نتائج كرة قدم للهواة",
    "amateur football right wingers": "أجنحة يمين كرة قدم للهواة",
    "amateur football rivalries": "دربيات كرة قدم للهواة",
    "amateur football running backs": "راكضون للخلف كرة قدم للهواة",
    "amateur football scouts": "كشافة كرة قدم للهواة",
    "amateur football shooting guards": "مدافعون مسددون كرة قدم للهواة",
    "amateur football small forwards": "مهاجمون صغيرو الجسم كرة قدم للهواة",
    "amateur football sports-people": "رياضيو كرة قدم للهواة",
    "amateur football squads": "تشكيلات كرة قدم للهواة",
    "amateur football statistics": "إحصائيات كرة قدم للهواة",
    "amateur football tackles": "مصطدمو كرة قدم للهواة",
    "amateur football teams": "فرق كرة قدم للهواة",
    "amateur football templates": "قوالب كرة قدم للهواة",
    "amateur football tournaments": "بطولات كرة قدم للهواة",
    "amateur football trainers": "مدربو كرة قدم للهواة",
    "amateur football umpires": "حكام كرة قدم للهواة",
    "amateur football utility players": "لاعبو مراكز متعددة كرة قدم للهواة",
    "amateur football venues": "ملاعب كرة قدم للهواة",
    "amateur football wide receivers": "مستقبلون واسعون كرة قدم للهواة",
    "amateur football wing halves": "أنصاف أجنحة كرة قدم للهواة",
    "amateur football wingers": "أجنحة كرة قدم للهواة",
    "mens youth football": "كرة قدم للشباب",
    "mens youth football centers": "لاعبو وسط كرة قدم للشباب",
    "mens youth football central defenders": "قلوب دفاع كرة قدم للشباب",
    "mens youth football centres": "لاعبو وسط كرة قدم للشباب",
    "mens youth football champions": "أبطال كرة قدم للشباب",
    "mens youth football clubs": "أندية كرة قدم للشباب",
    "mens youth football coaches": "مدربو كرة قدم للشباب",
    "mens youth football competitions": "منافسات كرة قدم للشباب",
    "mens youth football defencemen": "مدافعو كرة قدم للشباب",
    "mens youth football defenders": "مدافعو كرة قدم للشباب",
    "mens youth football defensive backs": "مدافعون خلفيون كرة قدم للشباب",
    "mens youth football defensive linemen": "مدافعو خط كرة قدم للشباب",
    "mens youth football directors": "مدراء كرة قدم للشباب",
    "mens youth football drop kickers": "مسددو ركلات كرة قدم للشباب",
    "mens youth football events": "أحداث كرة قدم للشباب",
    "mens youth football films": "أفلام كرة قدم للشباب",
    "mens youth football finals": "نهائيات كرة قدم للشباب",
    "mens youth football forwards": "مهاجمو كرة قدم للشباب",
    "mens youth football fullbacks": "مدافعو كرة قدم للشباب",
    "mens youth football goalkeepers": "حراس مرمى كرة قدم للشباب",
    "mens youth football goaltenders": "حراس مرمى كرة قدم للشباب",
    "mens youth football guards": "حراس كرة قدم للشباب",
    "mens youth football halfbacks": "أظهرة مساعدون كرة قدم للشباب",
    "mens youth football home stadiums": "ملاعب كرة قدم للشباب",
    "mens youth football inside forwards": "مهاجمون داخليون كرة قدم للشباب",
    "mens youth football journalists": "صحفيو كرة قدم للشباب",
    "mens youth football kickers": "راكلو كرة قدم للشباب",
    "mens youth football leagues": "دوريات كرة قدم للشباب",
    "mens youth football left wingers": "أجنحة يسار كرة قدم للشباب",
    "mens youth football linebackers": "أظهرة كرة قدم للشباب",
    "mens youth football lists": "قوائم كرة قدم للشباب",
    "mens youth football manager history": "تاريخ مدربو كرة قدم للشباب",
    "mens youth football managers": "مدربو كرة قدم للشباب",
    "mens youth football matches": "مباريات كرة قدم للشباب",
    "mens youth football midfielders": "لاعبو وسط كرة قدم للشباب",
    "mens youth football navigational boxes": "صناديق تصفح كرة قدم للشباب",
    "mens youth football non-profit organizations": "منظمات غير ربحية كرة قدم للشباب",
    "mens youth football non-profit publishers": "ناشرون غير ربحيون كرة قدم للشباب",
    "mens youth football offensive linemen": "مهاجمو خط كرة قدم للشباب",
    "mens youth football organisations": "منظمات كرة قدم للشباب",
    "mens youth football organizations": "منظمات كرة قدم للشباب",
    "mens youth football outside forwards": "مهاجمون خارجيون كرة قدم للشباب",
    "mens youth football peoplee": "أعلام كرة قدم للشباب",
    "mens youth football placekickers": "مسددو كرة قدم للشباب",
    "mens youth football players": "لاعبو كرة قدم للشباب",
    "mens youth football playerss": "لاعبو كرة قدم للشباب",
    "mens youth football point guards": "لاعبو هجوم خلفي كرة قدم للشباب",
    "mens youth football positions": "مراكز كرة قدم للشباب",
    "mens youth football power forwards": "مهاجمون أقوياء الجسم كرة قدم للشباب",
    "mens youth football quarterbacks": "أظهرة رباعيون كرة قدم للشباب",
    "mens youth football receivers": "مستقبلو كرة قدم للشباب",
    "mens youth football records": "سجلات كرة قدم للشباب",
    "mens youth football records and statistics": "سجلات وإحصائيات كرة قدم للشباب",
    "mens youth football referees": "حكام كرة قدم للشباب",
    "mens youth football results": "نتائج كرة قدم للشباب",
    "mens youth football right wingers": "أجنحة يمين كرة قدم للشباب",
    "mens youth football rivalries": "دربيات كرة قدم للشباب",
    "mens youth football running backs": "راكضون للخلف كرة قدم للشباب",
    "mens youth football scouts": "كشافة كرة قدم للشباب",
    "mens youth football shooting guards": "مدافعون مسددون كرة قدم للشباب",
    "mens youth football small forwards": "مهاجمون صغيرو الجسم كرة قدم للشباب",
    "mens youth football sports-people": "رياضيو كرة قدم للشباب",
    "mens youth football squads": "تشكيلات كرة قدم للشباب",
    "mens youth football statistics": "إحصائيات كرة قدم للشباب",
    "mens youth football tackles": "مصطدمو كرة قدم للشباب",
    "mens youth football teams": "فرق كرة قدم للشباب",
    "mens youth football templates": "قوالب كرة قدم للشباب",
    "mens youth football tournaments": "بطولات كرة قدم للشباب",
    "mens youth football trainers": "مدربو كرة قدم للشباب",
    "mens youth football umpires": "حكام كرة قدم للشباب",
    "mens youth football utility players": "لاعبو مراكز متعددة كرة قدم للشباب",
    "mens youth football venues": "ملاعب كرة قدم للشباب",
    "mens youth football wide receivers": "مستقبلون واسعون كرة قدم للشباب",
    "mens youth football wing halves": "أنصاف أجنحة كرة قدم للشباب",
    "mens youth football wingers": "أجنحة كرة قدم للشباب",
    "mens football": "كرة قدم رجالية",
    "mens football centers": "لاعبو وسط كرة قدم رجالية",
    "mens football central defenders": "قلوب دفاع كرة قدم رجالية",
    "mens football centres": "لاعبو وسط كرة قدم رجالية",
    "mens football champions": "أبطال كرة قدم رجالية",
    "mens football clubs": "أندية كرة قدم رجالية",
    "mens football coaches": "مدربو كرة قدم رجالية",
    "mens football competitions": "منافسات كرة قدم رجالية",
    "mens football defencemen": "مدافعو كرة قدم رجالية",
    "mens football defenders": "مدافعو كرة قدم رجالية",
    "mens football defensive backs": "مدافعون خلفيون كرة قدم رجالية",
    "mens football defensive linemen": "مدافعو خط كرة قدم رجالية",
    "mens football directors": "مدراء كرة قدم رجالية",
    "mens football drop kickers": "مسددو ركلات كرة قدم رجالية",
    "mens football events": "أحداث كرة قدم رجالية",
    "mens football films": "أفلام كرة قدم رجالية",
    "mens football finals": "نهائيات كرة قدم رجالية",
    "mens football forwards": "مهاجمو كرة قدم رجالية",
    "mens football fullbacks": "مدافعو كرة قدم رجالية",
    "mens football goalkeepers": "حراس مرمى كرة قدم رجالية",
    "mens football goaltenders": "حراس مرمى كرة قدم رجالية",
    "mens football guards": "حراس كرة قدم رجالية",
    "mens football halfbacks": "أظهرة مساعدون كرة قدم رجالية",
    "mens football home stadiums": "ملاعب كرة قدم رجالية",
    "mens football inside forwards": "مهاجمون داخليون كرة قدم رجالية",
    "mens football journalists": "صحفيو كرة قدم رجالية",
    "mens football kickers": "راكلو كرة قدم رجالية",
    "mens football leagues": "دوريات كرة قدم رجالية",
    "mens football left wingers": "أجنحة يسار كرة قدم رجالية",
    "mens football linebackers": "أظهرة كرة قدم رجالية",
    "mens football lists": "قوائم كرة قدم رجالية",
    "mens football manager history": "تاريخ مدربو كرة قدم رجالية",
    "mens football managers": "مدربو كرة قدم رجالية",
    "mens football matches": "مباريات كرة قدم رجالية",
    "mens football midfielders": "لاعبو وسط كرة قدم رجالية",
    "mens football navigational boxes": "صناديق تصفح كرة قدم رجالية",
    "mens football non-profit organizations": "منظمات غير ربحية كرة قدم رجالية",
    "mens football non-profit publishers": "ناشرون غير ربحيون كرة قدم رجالية",
    "mens football offensive linemen": "مهاجمو خط كرة قدم رجالية",
    "mens football organisations": "منظمات كرة قدم رجالية",
    "mens football organizations": "منظمات كرة قدم رجالية",
    "mens football outside forwards": "مهاجمون خارجيون كرة قدم رجالية",
    "mens football peoplee": "أعلام كرة قدم رجالية",
    "mens football placekickers": "مسددو كرة قدم رجالية",
    "mens football players": "لاعبو كرة قدم رجالية",
    "mens football playerss": "لاعبو كرة قدم رجالية",
    "mens football point guards": "لاعبو هجوم خلفي كرة قدم رجالية",
    "mens football positions": "مراكز كرة قدم رجالية",
    "mens football power forwards": "مهاجمون أقوياء الجسم كرة قدم رجالية",
    "mens football quarterbacks": "أظهرة رباعيون كرة قدم رجالية",
    "mens football receivers": "مستقبلو كرة قدم رجالية",
    "mens football records": "سجلات كرة قدم رجالية",
    "mens football records and statistics": "سجلات وإحصائيات كرة قدم رجالية",
    "mens football referees": "حكام كرة قدم رجالية",
    "mens football results": "نتائج كرة قدم رجالية",
    "mens football right wingers": "أجنحة يمين كرة قدم رجالية",
    "mens football rivalries": "دربيات كرة قدم رجالية",
    "mens football running backs": "راكضون للخلف كرة قدم رجالية",
    "mens football scouts": "كشافة كرة قدم رجالية",
    "mens football shooting guards": "مدافعون مسددون كرة قدم رجالية",
    "mens football small forwards": "مهاجمون صغيرو الجسم كرة قدم رجالية",
    "mens football sports-people": "رياضيو كرة قدم رجالية",
    "mens football squads": "تشكيلات كرة قدم رجالية",
    "mens football statistics": "إحصائيات كرة قدم رجالية",
    "mens football tackles": "مصطدمو كرة قدم رجالية",
    "mens football teams": "فرق كرة قدم رجالية",
    "mens football templates": "قوالب كرة قدم رجالية",
    "mens football tournaments": "بطولات كرة قدم رجالية",
    "mens football trainers": "مدربو كرة قدم رجالية",
    "mens football umpires": "حكام كرة قدم رجالية",
    "mens football utility players": "لاعبو مراكز متعددة كرة قدم رجالية",
    "mens football venues": "ملاعب كرة قدم رجالية",
    "mens football wide receivers": "مستقبلون واسعون كرة قدم رجالية",
    "mens football wing halves": "أنصاف أجنحة كرة قدم رجالية",
    "mens football wingers": "أجنحة كرة قدم رجالية",
    "youth football": "كرة قدم شبابية",
    "youth football centers": "لاعبو وسط كرة قدم شبابية",
    "youth football central defenders": "قلوب دفاع كرة قدم شبابية",
    "youth football centres": "لاعبو وسط كرة قدم شبابية",
    "youth football champions": "أبطال كرة قدم شبابية",
    "youth football clubs": "أندية كرة قدم شبابية",
    "youth football coaches": "مدربو كرة قدم شبابية",
    "youth football competitions": "منافسات كرة قدم شبابية",
    "youth football defencemen": "مدافعو كرة قدم شبابية",
    "youth football defenders": "مدافعو كرة قدم شبابية",
    "youth football defensive backs": "مدافعون خلفيون كرة قدم شبابية",
    "youth football defensive linemen": "مدافعو خط كرة قدم شبابية",
    "youth football directors": "مدراء كرة قدم شبابية",
    "youth football drop kickers": "مسددو ركلات كرة قدم شبابية",
    "youth football events": "أحداث كرة قدم شبابية",
    "youth football films": "أفلام كرة قدم شبابية",
    "youth football finals": "نهائيات كرة قدم شبابية",
    "youth football forwards": "مهاجمو كرة قدم شبابية",
    "youth football fullbacks": "مدافعو كرة قدم شبابية",
    "youth football goalkeepers": "حراس مرمى كرة قدم شبابية",
    "youth football goaltenders": "حراس مرمى كرة قدم شبابية",
    "youth football guards": "حراس كرة قدم شبابية",
    "youth football halfbacks": "أظهرة مساعدون كرة قدم شبابية",
    "youth football home stadiums": "ملاعب كرة قدم شبابية",
    "youth football inside forwards": "مهاجمون داخليون كرة قدم شبابية",
    "youth football journalists": "صحفيو كرة قدم شبابية",
    "youth football kickers": "راكلو كرة قدم شبابية",
    "youth football leagues": "دوريات كرة قدم شبابية",
    "youth football left wingers": "أجنحة يسار كرة قدم شبابية",
    "youth football linebackers": "أظهرة كرة قدم شبابية",
    "youth football lists": "قوائم كرة قدم شبابية",
    "youth football manager history": "تاريخ مدربو كرة قدم شبابية",
    "youth football managers": "مدربو كرة قدم شبابية",
    "youth football matches": "مباريات كرة قدم شبابية",
    "youth football midfielders": "لاعبو وسط كرة قدم شبابية",
    "youth football navigational boxes": "صناديق تصفح كرة قدم شبابية",
    "youth football non-profit organizations": "منظمات غير ربحية كرة قدم شبابية",
    "youth football non-profit publishers": "ناشرون غير ربحيون كرة قدم شبابية",
    "youth football offensive linemen": "مهاجمو خط كرة قدم شبابية",
    "youth football organisations": "منظمات كرة قدم شبابية",
    "youth football organizations": "منظمات كرة قدم شبابية",
    "youth football outside forwards": "مهاجمون خارجيون كرة قدم شبابية",
    "youth football peoplee": "أعلام كرة قدم شبابية",
    "youth football placekickers": "مسددو كرة قدم شبابية",
    "youth football players": "لاعبو كرة قدم شبابية",
    "youth football playerss": "لاعبو كرة قدم شبابية",
    "youth football point guards": "لاعبو هجوم خلفي كرة قدم شبابية",
    "youth football positions": "مراكز كرة قدم شبابية",
    "youth football power forwards": "مهاجمون أقوياء الجسم كرة قدم شبابية",
    "youth football quarterbacks": "أظهرة رباعيون كرة قدم شبابية",
    "youth football receivers": "مستقبلو كرة قدم شبابية",
    "youth football records": "سجلات كرة قدم شبابية",
    "youth football records and statistics": "سجلات وإحصائيات كرة قدم شبابية",
    "youth football referees": "حكام كرة قدم شبابية",
    "youth football results": "نتائج كرة قدم شبابية",
    "youth football right wingers": "أجنحة يمين كرة قدم شبابية",
    "youth football rivalries": "دربيات كرة قدم شبابية",
    "youth football running backs": "راكضون للخلف كرة قدم شبابية",
    "youth football scouts": "كشافة كرة قدم شبابية",
    "youth football shooting guards": "مدافعون مسددون كرة قدم شبابية",
    "youth football small forwards": "مهاجمون صغيرو الجسم كرة قدم شبابية",
    "youth football sports-people": "رياضيو كرة قدم شبابية",
    "youth football squads": "تشكيلات كرة قدم شبابية",
    "youth football statistics": "إحصائيات كرة قدم شبابية",
    "youth football tackles": "مصطدمو كرة قدم شبابية",
    "youth football teams": "فرق كرة قدم شبابية",
    "youth football templates": "قوالب كرة قدم شبابية",
    "youth football tournaments": "بطولات كرة قدم شبابية",
    "youth football trainers": "مدربو كرة قدم شبابية",
    "youth football umpires": "حكام كرة قدم شبابية",
    "youth football utility players": "لاعبو مراكز متعددة كرة قدم شبابية",
    "youth football venues": "ملاعب كرة قدم شبابية",
    "youth football wide receivers": "مستقبلون واسعون كرة قدم شبابية",
    "youth football wing halves": "أنصاف أجنحة كرة قدم شبابية",
    "youth football wingers": "أجنحة كرة قدم شبابية",
    "football centers": "لاعبو وسط كرة قدم",
    "football central defenders": "قلوب دفاع كرة قدم",
    "football centres": "لاعبو وسط كرة قدم",
    "football champions": "أبطال كرة قدم",
    "football clubs": "أندية كرة قدم",
    "football coaches": "مدربو كرة قدم",
    "football competitions": "منافسات كرة قدم",
    "football defencemen": "مدافعو كرة قدم",
    "football defenders": "مدافعو كرة قدم",
    "football defensive backs": "مدافعون خلفيون كرة قدم",
    "football defensive linemen": "مدافعو خط كرة قدم",
    "football directors": "مدراء كرة قدم",
    "football drop kickers": "مسددو ركلات كرة قدم",
    "football events": "أحداث كرة قدم",
    "football films": "أفلام كرة قدم",
    "football finals": "نهائيات كرة قدم",
    "football forwards": "مهاجمو كرة قدم",
    "football fullbacks": "مدافعو كرة قدم",
    "football goalkeepers": "حراس مرمى كرة قدم",
    "football goaltenders": "حراس مرمى كرة قدم",
    "football guards": "حراس كرة قدم",
    "football halfbacks": "أظهرة مساعدون كرة قدم",
    "football home stadiums": "ملاعب كرة قدم",
    "football inside forwards": "مهاجمون داخليون كرة قدم",
    "football journalists": "صحفيو كرة قدم",
    "football kickers": "راكلو كرة قدم",
    "football leagues": "دوريات كرة قدم",
    "football left wingers": "أجنحة يسار كرة قدم",
    "football linebackers": "أظهرة كرة قدم",
    "football lists": "قوائم كرة قدم",
    "football manager history": "تاريخ مدربو كرة قدم",
    "football managers": "مدربو كرة قدم",
    "football matches": "مباريات كرة قدم",
    "football midfielders": "لاعبو وسط كرة قدم",
    "football navigational boxes": "صناديق تصفح كرة قدم",
    "football non-profit organizations": "منظمات غير ربحية كرة قدم",
    "football non-profit publishers": "ناشرون غير ربحيون كرة قدم",
    "football offensive linemen": "مهاجمو خط كرة قدم",
    "football organisations": "منظمات كرة قدم",
    "football organizations": "منظمات كرة قدم",
    "football outside forwards": "مهاجمون خارجيون كرة قدم",
    "football people": "أعلام كرة قدم",
    "football peoplee": "أعلام كرة قدم",
    "football placekickers": "مسددو كرة قدم",
    "football players": "لاعبو كرة قدم",
    "football playerss": "لاعبو كرة قدم",
    "football point guards": "لاعبو هجوم خلفي كرة قدم",
    "football positions": "مراكز كرة قدم",
    "football power forwards": "مهاجمون أقوياء الجسم كرة قدم",
    "football quarterbacks": "أظهرة رباعيون كرة قدم",
    "football receivers": "مستقبلو كرة قدم",
    "football records": "سجلات كرة قدم",
    "football records and statistics": "سجلات وإحصائيات كرة قدم",
    "football referees": "حكام كرة قدم",
    "football results": "نتائج كرة قدم",
    "football right wingers": "أجنحة يمين كرة قدم",
    "football rivalries": "دربيات كرة قدم",
    "football running backs": "راكضون للخلف كرة قدم",
    "football scouts": "كشافة كرة قدم",
    "football shooting guards": "مدافعون مسددون كرة قدم",
    "football small forwards": "مهاجمون صغيرو الجسم كرة قدم",
    "football sports-people": "رياضيو كرة قدم",
    "football squads": "تشكيلات كرة قدم",
    "football statistics": "إحصائيات كرة قدم",
    "football tackles": "مصطدمو كرة قدم",
    "football teams": "فرق كرة قدم",
    "football templates": "قوالب كرة قدم",
    "football tournaments": "بطولات كرة قدم",
    "football trainers": "مدربو كرة قدم",
    "football umpires": "حكام كرة قدم",
    "football utility players": "لاعبو مراكز متعددة كرة قدم",
    "football venues": "ملاعب كرة قدم",
    "football wide receivers": "مستقبلون واسعون كرة قدم",
    "football wing halves": "أنصاف أجنحة كرة قدم",
    "football wingers": "أجنحة كرة قدم",
}


@pytest.mark.parametrize("category, expected_key", teams_2025_data.items(), ids=teams_2025_data.keys())
@pytest.mark.fast
def test_teams_2025_data(category: str, expected_key: str) -> None:
    label = find_teams_2025(category)
    assert label == expected_key


@pytest.mark.parametrize("category, expected_key", teams_2025_data_womens.items(), ids=teams_2025_data_womens.keys())
@pytest.mark.fast
def test_teams_2025_data_womens(category: str, expected_key: str) -> None:
    label = find_teams_2025(category)
    assert label == expected_key


@pytest.mark.parametrize("category, expected_key", data6.items(), ids=data6.keys())
@pytest.mark.fast
def test_find_teams_2025(category: str, expected_key: str) -> None:
    label = find_teams_2025(category)
    assert label == expected_key



"""

Usage:
from ...matables_bots.table1_bot import table1get, get_KAKO

"""

import functools
from typing import Dict

from ...helps import logger
from ...helps.jsonl_dump import dump_data
from ...new_resolvers.bys_new import resolve_by_labels
from ...translations import Jobs_new  # to be removed from players_new_keys
from ...translations import jobs_mens_data  # to be  removed from players_new_keys
from ...translations import (
    Films_key_man,
)
from ..make_bots.bot_2018 import pop_All_2018
from .bot import All_P17, Films_O_TT, players_new_keys

KAKO: Dict[str, Dict[str, str]] = {
    "pop_All_2018": pop_All_2018,  # 161
    "Films_key_man": Films_key_man,  # 74
    "All_P17": All_P17,  # 0
    "Films_O_TT": Films_O_TT,  # 0
    "players_new_keys": players_new_keys,  # 1,719
    "jobs_mens_data": jobs_mens_data,  # 96,552
    "Jobs_new": Jobs_new,  # 1,304
}


@functools.lru_cache(maxsize=None)
# @dump_data(1)
def _get_KAKO(text: str) -> str:
    """Look up the Arabic label for a term across several mapping tables."""
    resolved_label = resolve_by_labels(text)
    if resolved_label:
        return "resolve_by_labels", resolved_label

    for table_name, table_data in KAKO.items():
        resolved_label = table_data.get(text, "")
        if not resolved_label:
            continue

        # If not a string → also an error
        if not isinstance(resolved_label, str):
            raise TypeError(
                f"Resolver '{table_name}' returned non-string type {type(resolved_label)}: {resolved_label}"
            )

        logger.debug(f'>> get_KAKO_({table_name}) for ["{text}"] = "{resolved_label}"')

        return table_name, resolved_label

    return "", ""


@functools.lru_cache(maxsize=10000)
def get_KAKO(text: str) -> str:
    _, label = _get_KAKO(text)
    return label


__all__ = [
    "get_KAKO",
]



"""
Tests for bys_new bot
"""

import pytest
from load_one_data import dump_diff, one_dump_test

from ArWikiCats.new_resolvers.bys_new import resolve_by_labels

_by_of_fields = {
    "by century of completion": "حسب قرن الانتهاء",
    "by century of developer": "حسب قرن التطوير",
    "by century of disestablishment": "حسب قرن الانحلال",
    "by century of establishment": "حسب قرن التأسيس",
    "by century of formal description": "حسب قرن الوصف",
    "by century of introduction": "حسب قرن الاستحداث",
    "by century of invention": "حسب قرن الاختراع",
    "by century of location": "حسب قرن الموقع",
    "by century of photographing": "حسب قرن التصوير",
    "by century of reestablishment": "حسب قرن إعادة التأسيس",
    "by century of setting": "حسب قرن الأحداث",
    "by century of setting location": "حسب قرن موقع الأحداث",
    "by century of shooting location": "حسب قرن موقع التصوير",
    "by city of completion": "حسب مدينة الانتهاء",
    "by city of developer": "حسب مدينة التطوير",
    "by city of disestablishment": "حسب مدينة الانحلال",
    "by city of establishment": "حسب مدينة التأسيس",
    "by city of formal description": "حسب مدينة الوصف",
    "by city of introduction": "حسب مدينة الاستحداث",
    "by city of invention": "حسب مدينة الاختراع",
    "by city of location": "حسب مدينة الموقع",
    "by city of photographing": "حسب مدينة التصوير",
    "by city of reestablishment": "حسب مدينة إعادة التأسيس",
    "by city of setting": "حسب مدينة الأحداث",
    "by city of setting location": "حسب مدينة موقع الأحداث",
    "by city of shooting location": "حسب مدينة موقع التصوير",
    "by continent of completion": "حسب قارة الانتهاء",
    "by continent of developer": "حسب قارة التطوير",
    "by continent of disestablishment": "حسب قارة الانحلال",
    "by continent of establishment": "حسب قارة التأسيس",
    "by continent of formal description": "حسب قارة الوصف",
    "by continent of introduction": "حسب قارة الاستحداث",
    "by continent of invention": "حسب قارة الاختراع",
    "by continent of location": "حسب قارة الموقع",
    "by continent of photographing": "حسب قارة التصوير",
    "by continent of reestablishment": "حسب قارة إعادة التأسيس",
    "by continent of setting": "حسب قارة الأحداث",
    "by continent of setting location": "حسب قارة موقع الأحداث",
    "by continent of shooting location": "حسب قارة موقع التصوير",
    "by country of completion": "حسب بلد الانتهاء",
    "by country of developer": "حسب بلد التطوير",
    "by country of disestablishment": "حسب بلد الانحلال",
    "by country of establishment": "حسب بلد التأسيس",
    "by country of formal description": "حسب بلد الوصف",
    "by country of introduction": "حسب بلد الاستحداث",
    "by country of invention": "حسب بلد الاختراع",
    "by country of location": "حسب بلد الموقع",
    "by country of photographing": "حسب بلد التصوير",
    "by country of reestablishment": "حسب بلد إعادة التأسيس",
    "by country of setting": "حسب بلد الأحداث",
    "by country of setting location": "حسب بلد موقع الأحداث",
    "by country of shooting location": "حسب بلد موقع التصوير",
    "by date of completion": "حسب تاريخ الانتهاء",
    "by date of developer": "حسب تاريخ التطوير",
    "by date of disestablishment": "حسب تاريخ الانحلال",
    "by date of establishment": "حسب تاريخ التأسيس",
    "by date of formal description": "حسب تاريخ الوصف",
    "by date of introduction": "حسب تاريخ الاستحداث",
    "by date of invention": "حسب تاريخ الاختراع",
    "by date of location": "حسب تاريخ الموقع",
    "by date of photographing": "حسب تاريخ التصوير",
    "by date of reestablishment": "حسب تاريخ إعادة التأسيس",
    "by date of setting": "حسب تاريخ الأحداث",
    "by date of setting location": "حسب تاريخ موقع الأحداث",
    "by date of shooting location": "حسب تاريخ موقع التصوير",
    "by decade of completion": "حسب عقد الانتهاء",
    "by decade of developer": "حسب عقد التطوير",
    "by decade of disestablishment": "حسب عقد الانحلال",
    "by decade of establishment": "حسب عقد التأسيس",
    "by decade of formal description": "حسب عقد الوصف",
    "by decade of introduction": "حسب عقد الاستحداث",
    "by decade of invention": "حسب عقد الاختراع",
    "by decade of location": "حسب عقد الموقع",
    "by decade of photographing": "حسب عقد التصوير",
    "by decade of reestablishment": "حسب عقد إعادة التأسيس",
    "by decade of setting": "حسب عقد الأحداث",
    "by decade of setting location": "حسب عقد موقع الأحداث",
    "by decade of shooting location": "حسب عقد موقع التصوير",
    "by era of completion": "حسب عصر الانتهاء",
    "by era of developer": "حسب عصر التطوير",
    "by era of disestablishment": "حسب عصر الانحلال",
    "by era of establishment": "حسب عصر التأسيس",
    "by era of formal description": "حسب عصر الوصف",
    "by era of introduction": "حسب عصر الاستحداث",
    "by era of invention": "حسب عصر الاختراع",
    "by era of location": "حسب عصر الموقع",
    "by era of photographing": "حسب عصر التصوير",
    "by era of reestablishment": "حسب عصر إعادة التأسيس",
    "by era of setting": "حسب عصر الأحداث",
    "by era of setting location": "حسب عصر موقع الأحداث",
    "by era of shooting location": "حسب عصر موقع التصوير",
    "by location of completion": "حسب موقع الانتهاء",
    "by location of developer": "حسب موقع التطوير",
    "by location of disestablishment": "حسب موقع الانحلال",
    "by location of establishment": "حسب موقع التأسيس",
    "by location of formal description": "حسب موقع الوصف",
    "by location of introduction": "حسب موقع الاستحداث",
    "by location of invention": "حسب موقع الاختراع",
    "by location of location": "حسب موقع الموقع",
    "by location of photographing": "حسب موقع التصوير",
    "by location of reestablishment": "حسب موقع إعادة التأسيس",
    "by location of setting": "حسب موقع الأحداث",
    "by location of setting location": "حسب موقع موقع الأحداث",
    "by location of shooting location": "حسب موقع موقع التصوير",
    "by millennium of completion": "حسب ألفية الانتهاء",
    "by millennium of developer": "حسب ألفية التطوير",
    "by millennium of disestablishment": "حسب ألفية الانحلال",
    "by millennium of establishment": "حسب ألفية التأسيس",
    "by millennium of formal description": "حسب ألفية الوصف",
    "by millennium of introduction": "حسب ألفية الاستحداث",
    "by millennium of invention": "حسب ألفية الاختراع",
    "by millennium of location": "حسب ألفية الموقع",
    "by millennium of photographing": "حسب ألفية التصوير",
    "by millennium of reestablishment": "حسب ألفية إعادة التأسيس",
    "by millennium of setting": "حسب ألفية الأحداث",
    "by millennium of setting location": "حسب ألفية موقع الأحداث",
    "by millennium of shooting location": "حسب ألفية موقع التصوير",
    "by period of completion": "حسب حقبة الانتهاء",
    "by period of developer": "حسب حقبة التطوير",
    "by period of disestablishment": "حسب حقبة الانحلال",
    "by period of establishment": "حسب حقبة التأسيس",
    "by period of formal description": "حسب حقبة الوصف",
    "by period of introduction": "حسب حقبة الاستحداث",
    "by period of invention": "حسب حقبة الاختراع",
    "by period of location": "حسب حقبة الموقع",
    "by period of photographing": "حسب حقبة التصوير",
    "by period of reestablishment": "حسب حقبة إعادة التأسيس",
    "by period of setting": "حسب حقبة الأحداث",
    "by period of setting location": "حسب حقبة موقع الأحداث",
    "by period of shooting location": "حسب حقبة موقع التصوير",
    "by time of completion": "حسب وقت الانتهاء",
    "by time of developer": "حسب وقت التطوير",
    "by time of disestablishment": "حسب وقت الانحلال",
    "by time of establishment": "حسب وقت التأسيس",
    "by time of formal description": "حسب وقت الوصف",
    "by time of introduction": "حسب وقت الاستحداث",
    "by time of invention": "حسب وقت الاختراع",
    "by time of location": "حسب وقت الموقع",
    "by time of photographing": "حسب وقت التصوير",
    "by time of reestablishment": "حسب وقت إعادة التأسيس",
    "by time of setting": "حسب وقت الأحداث",
    "by time of setting location": "حسب وقت موقع الأحداث",
    "by time of shooting location": "حسب وقت موقع التصوير",
    "by year of completion": "حسب سنة الانتهاء",
    "by year of developer": "حسب سنة التطوير",
    "by year of disestablishment": "حسب سنة الانحلال",
    "by year of establishment": "حسب سنة التأسيس",
    "by year of formal description": "حسب سنة الوصف",
    "by year of introduction": "حسب سنة الاستحداث",
    "by year of invention": "حسب سنة الاختراع",
    "by year of location": "حسب سنة الموقع",
    "by year of photographing": "حسب سنة التصوير",
    "by year of reestablishment": "حسب سنة إعادة التأسيس",
    "by year of setting": "حسب سنة الأحداث",
    "by year of setting location": "حسب سنة موقع الأحداث",
    "by year of shooting location": "حسب سنة موقع التصوير",
}

_by_table_year = {
    "by year - boys doubles": "حسب السنة - زوجي فتيان",
    "by year - boys qualification": "حسب السنة - تصفيات فتيان",
    "by year - boys singles": "حسب السنة - فردي فتيان",
    "by year - boys team": "حسب السنة - فريق فتيان",
    "by year - boys tournament": "حسب السنة - مسابقة فتيان",
    "by year - girls doubles": "حسب السنة - زوجي فتيات",
    "by year - girls qualification": "حسب السنة - تصفيات فتيات",
    "by year - girls singles": "حسب السنة - فردي فتيات",
    "by year - girls team": "حسب السنة - فريق فتيات",
    "by year - girls tournament": "حسب السنة - مسابقة فتيات",
    "by year - ladies doubles": "حسب السنة - زوجي سيدات",
    "by year - ladies qualification": "حسب السنة - تصفيات سيدات",
    "by year - ladies singles": "حسب السنة - فردي سيدات",
    "by year - ladies team": "حسب السنة - فريق سيدات",
    "by year - ladies tournament": "حسب السنة - مسابقة سيدات",
    "by year - males doubles": "حسب السنة - زوجي رجال",
    "by year - males qualification": "حسب السنة - تصفيات رجال",
    "by year - males singles": "حسب السنة - فردي رجال",
    "by year - males team": "حسب السنة - فريق رجال",
    "by year - males tournament": "حسب السنة - مسابقة رجال",
    "by year - men's doubles": "حسب السنة - زوجي رجال",
    "by year - men's qualification": "حسب السنة - تصفيات رجال",
    "by year - men's singles": "حسب السنة - فردي رجال",
    "by year - men's team": "حسب السنة - فريق رجال",
    "by year - men's tournament": "حسب السنة - مسابقة رجال",
    "by year - mixed doubles": "حسب السنة - زوجي مختلط",
    "by year - mixed qualification": "حسب السنة - تصفيات مختلط",
    "by year - mixed singles": "حسب السنة - فردي مختلط",
    "by year - mixed team": "حسب السنة - فريق مختلط",
    "by year - mixed tournament": "حسب السنة - مسابقة مختلط",
    "by year - singles doubles": "حسب السنة - زوجي فردي",
    "by year - singles qualification": "حسب السنة - تصفيات فردي",
    "by year - singles singles": "حسب السنة - فردي فردي",
    "by year - singles team": "حسب السنة - فريق فردي",
    "by year - singles tournament": "حسب السنة - مسابقة فردي",
    "by year - womens doubles": "حسب السنة - زوجي سيدات",
    "by year - womens qualification": "حسب السنة - تصفيات سيدات",
    "by year - womens singles": "حسب السنة - فردي سيدات",
    "by year - womens team": "حسب السنة - فريق سيدات",
    "by year - womens tournament": "حسب السنة - مسابقة سيدات",
}

data1 = {
    "by populated place and decade": "حسب المكان المأهول والعقد",
    "by populated place and year": "حسب المكان المأهول والسنة",
    "by city or city of sport": "حسب المدينة أو مدينة الرياضة",
    "by city by city of sport": "حسب المدينة حسب مدينة الرياضة",
    "by city and city of sport": "حسب المدينة ومدينة الرياضة",
    "by country and country": "حسب البلد والبلد",
    "by country and city of setting": "حسب البلد ومدينة الأحداث",
    "by country and city of developer": "حسب البلد ومدينة التطوير",
    "by city": "حسب المدينة",
    "by country": "حسب البلد",
    "by year": "حسب السنة",
    "by city of developer": "حسب مدينة التطوير",
    "by organization": "حسب المنظمة",
    "by nonprofit organization": "حسب المنظمات غير الربحية",
    "by organization or nonprofit organization": "حسب المنظمة أو المنظمات غير الربحية",
    "by organization by nonprofit organization": "حسب المنظمة حسب المنظمات غير الربحية",
    "by organization and nonprofit organization": "حسب المنظمة والمنظمات غير الربحية",
}

data_yearly = {
    "by year - boys doubles": "حسب السنة - زوجي فتيان",
    "by year - boys qualification": "حسب السنة - تصفيات فتيان",
    "by year - boys singles": "حسب السنة - فردي فتيان",
    "by year - boys team": "حسب السنة - فريق فتيان",
    "by year - boys tournament": "حسب السنة - مسابقة فتيان",
    "by year - girls doubles": "حسب السنة - زوجي فتيات",
    "by year - girls qualification": "حسب السنة - تصفيات فتيات",
    "by year - girls singles": "حسب السنة - فردي فتيات",
    "by year - girls team": "حسب السنة - فريق فتيات",
    "by year - girls tournament": "حسب السنة - مسابقة فتيات",
    "by year - ladies doubles": "حسب السنة - زوجي سيدات",
    "by year - ladies qualification": "حسب السنة - تصفيات سيدات",
    "by year - ladies singles": "حسب السنة - فردي سيدات",
    "by year - ladies team": "حسب السنة - فريق سيدات",
    "by year - ladies tournament": "حسب السنة - مسابقة سيدات",
    "by year - males doubles": "حسب السنة - زوجي رجال",
    "by year - males qualification": "حسب السنة - تصفيات رجال",
    "by year - males singles": "حسب السنة - فردي رجال",
    "by year - males team": "حسب السنة - فريق رجال",
    "by year - males tournament": "حسب السنة - مسابقة رجال",
    "by year - men's doubles": "حسب السنة - زوجي رجال",
    "by year - men's qualification": "حسب السنة - تصفيات رجال",
    "by year - men's singles": "حسب السنة - فردي رجال",
    "by year - men's team": "حسب السنة - فريق رجال",
    "by year - men's tournament": "حسب السنة - مسابقة رجال",
    "by year - mixed doubles": "حسب السنة - زوجي مختلط",
    "by year - mixed qualification": "حسب السنة - تصفيات مختلط",
    "by year - mixed singles": "حسب السنة - فردي مختلط",
    "by year - mixed team": "حسب السنة - فريق مختلط",
    "by year - mixed tournament": "حسب السنة - مسابقة مختلط",
    "by year - singles doubles": "حسب السنة - زوجي فردي",
    "by year - singles qualification": "حسب السنة - تصفيات فردي",
    "by year - singles singles": "حسب السنة - فردي فردي",
    "by year - singles team": "حسب السنة - فريق فردي",
    "by year - singles tournament": "حسب السنة - مسابقة فردي",
    "by year - womens doubles": "حسب السنة - زوجي سيدات",
    "by year - womens qualification": "حسب السنة - تصفيات سيدات",
    "by year - womens singles": "حسب السنة - فردي سيدات",
    "by year - womens team": "حسب السنة - فريق سيدات",
    "by year - womens tournament": "حسب السنة - مسابقة سيدات",
}

by_keys_under = {
    "by men's under-16 national team": "حسب المنتخب الوطني للرجال تحت 16 سنة",
    "by men's under-17 national team": "حسب المنتخب الوطني للرجال تحت 17 سنة",
    "by men's under-18 national team": "حسب المنتخب الوطني للرجال تحت 18 سنة",
    "by men's under-19 national team": "حسب المنتخب الوطني للرجال تحت 19 سنة",
    "by men's under-20 national team": "حسب المنتخب الوطني للرجال تحت 20 سنة",
    "by men's under-21 national team": "حسب المنتخب الوطني للرجال تحت 21 سنة",
    "by men's under-23 national team": "حسب المنتخب الوطني للرجال تحت 23 سنة",
    "by under-16 national team": "حسب المنتخب الوطني تحت 16 سنة",
    "by under-17 national team": "حسب المنتخب الوطني تحت 17 سنة",
    "by under-18 national team": "حسب المنتخب الوطني تحت 18 سنة",
    "by under-19 national team": "حسب المنتخب الوطني تحت 19 سنة",
    "by under-20 national team": "حسب المنتخب الوطني تحت 20 سنة",
    "by under-21 national team": "حسب المنتخب الوطني تحت 21 سنة",
    "by under-23 national team": "حسب المنتخب الوطني تحت 23 سنة",
    "by women's under-16 national team": "حسب المنتخب الوطني للسيدات تحت 16 سنة",
    "by women's under-17 national team": "حسب المنتخب الوطني للسيدات تحت 17 سنة",
    "by women's under-18 national team": "حسب المنتخب الوطني للسيدات تحت 18 سنة",
    "by women's under-19 national team": "حسب المنتخب الوطني للسيدات تحت 19 سنة",
    "by women's under-20 national team": "حسب المنتخب الوطني للسيدات تحت 20 سنة",
    "by women's under-21 national team": "حسب المنتخب الوطني للسيدات تحت 21 سنة",
    "by women's under-23 national team": "حسب المنتخب الوطني للسيدات تحت 23 سنة",
}

by_table_all = {
    "by country and country subdivision": "حسب البلد وتقسيم البلد",
    "by country and country subdivisions": "حسب البلد وتقسيمات البلد",
    "by country and country-of-residence": "حسب البلد وبلد الإقامة",
    "by company and shipbuilding company": "حسب الشركة وشركة بناء السفن",
    "by organization and research organization": "حسب المنظمة ومنظمة البحوث",
    "by orientation and political orientation": "حسب التوجه والتوجه السياسي",
    "by religion and former religion": "حسب الدين والدين السابق",
    "by subdivision and country subdivision": "حسب التقسيم وتقسيم البلد",
    "by country or country subdivision": "حسب البلد أو تقسيم البلد",
    "by country or country subdivisions": "حسب البلد أو تقسيمات البلد",
    "by country or country-of-residence": "حسب البلد أو بلد الإقامة",
    "by company or shipbuilding company": "حسب الشركة أو شركة بناء السفن",
    "by organization or research organization": "حسب المنظمة أو منظمة البحوث",
    "by orientation or political orientation": "حسب التوجه أو التوجه السياسي",
    "by religion or former religion": "حسب الدين أو الدين السابق",
    "by subdivision or country subdivision": "حسب التقسيم أو تقسيم البلد",
    "by country by country subdivision": "حسب البلد حسب تقسيم البلد",
    "by country by country subdivisions": "حسب البلد حسب تقسيمات البلد",
    "by country by country-of-residence": "حسب البلد حسب بلد الإقامة",
    "by company by shipbuilding company": "حسب الشركة حسب شركة بناء السفن",
    "by organization by research organization": "حسب المنظمة حسب منظمة البحوث",
    "by orientation by political orientation": "حسب التوجه حسب التوجه السياسي",
    "by religion by former religion": "حسب الدين حسب الدين السابق",
    "by subdivision by country subdivision": "حسب التقسيم حسب تقسيم البلد",
    "by city of shooting location": "حسب مدينة موقع التصوير",
    "by continent of shooting location": "حسب قارة موقع التصوير",
    "by country of shooting location": "حسب بلد موقع التصوير",
    "by city of developer": "حسب مدينة التطوير",
    "by city of disestablishment": "حسب مدينة الانحلال",
    "by city of reestablishment": "حسب مدينة إعادة التأسيس",
    "by city of establishment": "حسب مدينة التأسيس",
    "by city of setting location": "حسب مدينة موقع الأحداث",
    "by city of invention": "حسب مدينة الاختراع",
    "by city of introduction": "حسب مدينة الاستحداث",
    "by city of formal description": "حسب مدينة الوصف",
    "by city of photographing": "حسب مدينة التصوير",
    "by city of completion": "حسب مدينة الانتهاء",
    "by date of shooting location": "حسب تاريخ موقع التصوير",
    "by date of developer": "حسب تاريخ التطوير",
    "by date of location": "حسب تاريخ الموقع",
    "by date of setting": "حسب تاريخ الأحداث",
    "by date of disestablishment": "حسب تاريخ الانحلال",
    "by date of reestablishment": "حسب تاريخ إعادة التأسيس",
    "by date of establishment": "حسب تاريخ التأسيس",
    "by date of setting location": "حسب تاريخ موقع الأحداث",
    "by date of invention": "حسب تاريخ الاختراع",
    "by date of introduction": "حسب تاريخ الاستحداث",
    "by date of formal description": "حسب تاريخ الوصف",
    "by date of photographing": "حسب تاريخ التصوير",
    "by date of completion": "حسب تاريخ الانتهاء",
    "by country of developer": "حسب بلد التطوير",
    "by country of disestablishment": "حسب بلد الانحلال",
    "by country of reestablishment": "حسب بلد إعادة التأسيس",
    "by country of establishment": "حسب بلد التأسيس",
    "by country of invention": "حسب بلد الاختراع",
    "by country of introduction": "حسب بلد الاستحداث",
    "by country of formal description": "حسب بلد الوصف",
    "by country of photographing": "حسب بلد التصوير",
    "by country of completion": "حسب بلد الانتهاء",
    "by continent of developer": "حسب قارة التطوير",
    "by continent of location": "حسب قارة الموقع",
    "by continent of disestablishment": "حسب قارة الانحلال",
    "by continent of reestablishment": "حسب قارة إعادة التأسيس",
    "by continent of establishment": "حسب قارة التأسيس",
    "by continent of setting location": "حسب قارة موقع الأحداث",
    "by continent of invention": "حسب قارة الاختراع",
    "by continent of introduction": "حسب قارة الاستحداث",
    "by continent of formal description": "حسب قارة الوصف",
    "by continent of photographing": "حسب قارة التصوير",
    "by continent of completion": "حسب قارة الانتهاء",
    "by location of shooting location": "حسب موقع موقع التصوير",
    "by location of developer": "حسب موقع التطوير",
    "by location of location": "حسب موقع الموقع",
    "by location of setting": "حسب موقع الأحداث",
    "by location of disestablishment": "حسب موقع الانحلال",
    "by location of reestablishment": "حسب موقع إعادة التأسيس",
    "by location of establishment": "حسب موقع التأسيس",
    "by location of setting location": "حسب موقع موقع الأحداث",
    "by location of invention": "حسب موقع الاختراع",
    "by location of introduction": "حسب موقع الاستحداث",
    "by location of formal description": "حسب موقع الوصف",
    "by location of photographing": "حسب موقع التصوير",
    "by location of completion": "حسب موقع الانتهاء",
    "by period of shooting location": "حسب حقبة موقع التصوير",
    "by period of developer": "حسب حقبة التطوير",
    "by period of location": "حسب حقبة الموقع",
    "by period of disestablishment": "حسب حقبة الانحلال",
    "by period of reestablishment": "حسب حقبة إعادة التأسيس",
    "by period of establishment": "حسب حقبة التأسيس",
    "by period of invention": "حسب حقبة الاختراع",
    "by period of introduction": "حسب حقبة الاستحداث",
    "by period of formal description": "حسب حقبة الوصف",
    "by period of photographing": "حسب حقبة التصوير",
    "by period of completion": "حسب حقبة الانتهاء",
    "by time of shooting location": "حسب وقت موقع التصوير",
    "by time of developer": "حسب وقت التطوير",
    "by time of location": "حسب وقت الموقع",
    "by time of setting": "حسب وقت الأحداث",
    "by time of disestablishment": "حسب وقت الانحلال",
    "by time of reestablishment": "حسب وقت إعادة التأسيس",
    "by time of establishment": "حسب وقت التأسيس",
    "by time of setting location": "حسب وقت موقع الأحداث",
    "by time of invention": "حسب وقت الاختراع",
    "by time of introduction": "حسب وقت الاستحداث",
    "by time of formal description": "حسب وقت الوصف",
    "by time of photographing": "حسب وقت التصوير",
    "by time of completion": "حسب وقت الانتهاء",
    "by year of shooting location": "حسب سنة موقع التصوير",
    "by year of developer": "حسب سنة التطوير",
    "by year of location": "حسب سنة الموقع",
    "by year of setting": "حسب سنة الأحداث",
    "by year of disestablishment": "حسب سنة الانحلال",
    "by year of reestablishment": "حسب سنة إعادة التأسيس",
    "by year of establishment": "حسب سنة التأسيس",
    "by year of setting location": "حسب سنة موقع الأحداث",
    "by year of invention": "حسب سنة الاختراع",
    "by year of formal description": "حسب سنة الوصف",
    "by decade of shooting location": "حسب عقد موقع التصوير",
    "by decade of developer": "حسب عقد التطوير",
    "by decade of location": "حسب عقد الموقع",
    "by decade of setting": "حسب عقد الأحداث",
    "by decade of disestablishment": "حسب عقد الانحلال",
    "by decade of reestablishment": "حسب عقد إعادة التأسيس",
    "by decade of establishment": "حسب عقد التأسيس",
    "by decade of setting location": "حسب عقد موقع الأحداث",
    "by decade of invention": "حسب عقد الاختراع",
    "by decade of formal description": "حسب عقد الوصف",
    "by decade of photographing": "حسب عقد التصوير",
    "by decade of completion": "حسب عقد الانتهاء",
    "by era of shooting location": "حسب عصر موقع التصوير",
    "by era of developer": "حسب عصر التطوير",
    "by era of location": "حسب عصر الموقع",
    "by era of setting": "حسب عصر الأحداث",
    "by era of disestablishment": "حسب عصر الانحلال",
    "by era of reestablishment": "حسب عصر إعادة التأسيس",
    "by era of establishment": "حسب عصر التأسيس",
    "by era of setting location": "حسب عصر موقع الأحداث",
    "by era of invention": "حسب عصر الاختراع",
    "by era of introduction": "حسب عصر الاستحداث",
    "by era of formal description": "حسب عصر الوصف",
    "by era of photographing": "حسب عصر التصوير",
    "by era of completion": "حسب عصر الانتهاء",
    "by millennium of shooting location": "حسب ألفية موقع التصوير",
    "by millennium of developer": "حسب ألفية التطوير",
    "by millennium of location": "حسب ألفية الموقع",
    "by millennium of setting": "حسب ألفية الأحداث",
    "by millennium of disestablishment": "حسب ألفية الانحلال",
    "by millennium of reestablishment": "حسب ألفية إعادة التأسيس",
    "by millennium of establishment": "حسب ألفية التأسيس",
    "by millennium of setting location": "حسب ألفية موقع الأحداث",
    "by millennium of invention": "حسب ألفية الاختراع",
    "by millennium of introduction": "حسب ألفية الاستحداث",
    "by millennium of formal description": "حسب ألفية الوصف",
    "by millennium of photographing": "حسب ألفية التصوير",
    "by millennium of completion": "حسب ألفية الانتهاء",
    "by century of shooting location": "حسب قرن موقع التصوير",
    "by century of developer": "حسب قرن التطوير",
    "by century of location": "حسب قرن الموقع",
    "by century of setting": "حسب قرن الأحداث",
    "by century of disestablishment": "حسب قرن الانحلال",
    "by century of reestablishment": "حسب قرن إعادة التأسيس",
    "by century of establishment": "حسب قرن التأسيس",
    "by century of setting location": "حسب قرن موقع الأحداث",
    "by century of invention": "حسب قرن الاختراع",
    "by century of introduction": "حسب قرن الاستحداث",
    "by century of formal description": "حسب قرن الوصف",
    "by century of photographing": "حسب قرن التصوير",
    "by century of completion": "حسب قرن الانتهاء",
}

to_test = [
    ("test_bys_all", by_table_all),
    ("test_bys_new_1", data1),
    ("test_by_keys_under", by_keys_under),
    ("test_data_yearly", data_yearly),
]


@pytest.mark.parametrize("category, expected", data_yearly.items(), ids=data_yearly.keys())
@pytest.mark.fast
def test_data_yearly(category: str, expected: str) -> None:
    label = resolve_by_labels(category)
    assert label == expected, f"Failed for category: {category}"


@pytest.mark.parametrize("category, expected", by_keys_under.items(), ids=by_keys_under.keys())
@pytest.mark.fast
def test_by_keys_under(category: str, expected: str) -> None:
    label = resolve_by_labels(category)
    assert label == expected, f"Failed for category: {category}"


@pytest.mark.parametrize("category, expected", by_table_all.items(), ids=by_table_all.keys())
@pytest.mark.fast
def test_bys_all(category: str, expected: str) -> None:
    label = resolve_by_labels(category)
    assert label == expected, f"Failed for category: {category}"


@pytest.mark.parametrize("category, expected", data1.items(), ids=data1.keys())
@pytest.mark.fast
def test_bys_new_1(category: str, expected: str) -> None:
    label = resolve_by_labels(category)
    assert label == expected, f"Failed for category: {category}"


@pytest.mark.parametrize("category, expected", _by_table_year.items(), ids=_by_table_year.keys())
@pytest.mark.slow
def test_by_table_year(category: str, expected: str) -> None:
    label = resolve_by_labels(category)
    assert label == expected, f"Failed for category: {category}"


@pytest.mark.parametrize("category, expected", _by_of_fields.items(), ids=_by_of_fields.keys())
@pytest.mark.slow
def test_by_of_fields(category: str, expected: str) -> None:
    label = resolve_by_labels(category)
    assert label == expected, f"Failed for category: {category}"


@pytest.mark.parametrize("name,data", to_test)
@pytest.mark.dump
def test_dump_all(name: str, data: dict[str, str]) -> None:
    expected, diff_result = one_dump_test(data, resolve_by_labels)
    dump_diff(diff_result, name)
    assert diff_result == expected, f"Differences found: {len(diff_result):,}, len all :{len(data):,}"



#!/usr/bin/python3
"""
Base module for category translation formatter classes.

This module provides the FormatDataBase class which serves as the foundation
for all single-element category translation formatters. It contains shared
functionality for pattern matching, template lookup, and placeholder replacement.

Classes:
    FormatDataBase: Abstract base class for all FormatData-type formatters.

The FormatDataBase class provides:
    - Regex pattern building from data_list keys
    - Case-insensitive key matching and template lookup
    - Placeholder normalization and replacement
    - Caching for performance optimization

Example:
    >>> # FormatDataBase is an abstract class - use FormatData instead
    >>> from ArWikiCats.translations_formats.DataModel import FormatData
    >>> bot = FormatData(
    ...     formatted_data={"{sport} players": "لاعبو {sport_label}"},
    ...     data_list={"football": "كرة القدم"},
    ...     key_placeholder="{sport}",
    ...     value_placeholder="{sport_label}",
    ... )
    >>> bot.search("football players")
    'لاعبو كرة القدم'
"""

import functools
import re
from typing import Any, Dict, Optional, Union

from ..formats_logger import logger


class FormatDataBase:
    """
    Abstract base class for single-element category translation formatters.

    This class provides the core functionality for translating category strings
    by matching keys from a data_list and replacing them using template patterns.
    It is meant to be subclassed by specific formatter implementations.

    Attributes:
        formatted_data (Dict[str, str]): Template patterns mapping English patterns to Arabic templates.
        formatted_data_ci (Dict[str, str]): Case-insensitive version of formatted_data.
        data_list (Dict[str, Any]): Key-to-Arabic-label mappings for replacements.
        data_list_ci (Dict[str, Any]): Case-insensitive version of data_list.
        key_placeholder (str): Placeholder string for the key in patterns.
        text_after (str): Optional text that appears after the key.
        text_before (str): Optional text that appears before the key.
        regex_filter (str): Regex pattern for word boundary detection.
        alternation (str): Regex alternation string built from data_list keys.
        pattern (re.Pattern): Compiled regex pattern for key matching.

    Methods:
        match_key: Find and return a matching key from the category.
        normalize_category: Replace matched key with placeholder.
        get_template: Get Arabic template for a category.
        get_template_ar: Get Arabic template by normalized key.
        get_key_label: Get Arabic label for a key.
        search: End-to-end translation of a category string.
        create_label: Alias for search.

    Note:
        Subclasses must implement apply_pattern_replacement and replace_value_placeholder.

    Example:
        >>> # This is an abstract class - see FormatData for usage
        >>> from ArWikiCats.translations_formats.DataModel import FormatData
    """

    def __init__(
        self,
        formatted_data: Dict[str, str],
        data_list: Dict[str, Any],
        key_placeholder: str = "xoxo",
        text_after: str = "",
        text_before: str = "",
        regex_filter: str = r"\w",
    ) -> None:
        """Prepare helpers for matching and formatting template-driven labels."""
        # Store originals
        self.formatted_data = formatted_data
        self.data_list = data_list
        self.text_after = text_after
        self.regex_filter = regex_filter or r"\w"  # [\w-]
        self.text_before = text_before

        # Case-insensitive mirrors
        self.formatted_data_ci: Dict[str, str] = {k.lower(): v for k, v in formatted_data.items()}
        self.data_list_ci: Dict[str, Any] = {k.lower(): v for k, v in data_list.items()}

        self.key_placeholder = key_placeholder
        self.alternation: str = None
        self.pattern: Optional[re.Pattern[str]] = None
        self.pattern_double: Optional[re.Pattern[str]] = None

    def add_formatted_data(self, key: str, value: str) -> None:
        """Add a key-value pair to the data_list."""
        self.formatted_data[key] = value
        self.formatted_data_ci[key.lower()] = value

    def create_alternation(self) -> str:
        """Create regex alternation from data_list_ci keys."""
        if not self.data_list_ci:
            return ""

        if len(self.data_list_ci) > 1000:
            logger.debug(f">keys_to_pattern(): len(new_pattern keys) = {len(self.data_list_ci):,}")

        # to fix bug that selected "black" instead of "black-and-white"
        keys_sorted = sorted(self.data_list_ci.keys(), key=lambda k: (-k.count(" "), -len(k)))

        return "|".join(map(re.escape, keys_sorted))

    def keys_to_pattern(self) -> Optional[re.Pattern[str]]:
        """
        Build a case-insensitive regex over lowercased keys of data_list.
        """
        if not self.data_list_ci:
            return None

        if self.alternation is None:
            self.alternation = self.create_alternation()

        data_pattern = rf"(?<!{self.regex_filter})({self.alternation})(?!{self.regex_filter})"
        return re.compile(data_pattern, re.I)

    @functools.lru_cache(maxsize=None)
    def match_key(self, category: str) -> str:
        """Return canonical lowercased key from data_list if found; else empty."""
        if not self.pattern:
            return ""
        # Normalize the category by removing extra spaces
        normalized_category = " ".join(category.split())
        logger.debug(f">!> match_key: {normalized_category=}")

        # TODO: check this
        if self.data_list_ci.get(normalized_category.lower()):
            logger.debug(f">>!!>> match_key: found in data_list_ci {normalized_category=}")
            return normalized_category.lower()

        match = self.pattern.search(f" {normalized_category} ")
        result = match.group(1).lower() if match else ""
        logger.debug(f"==== match_key {result=}")
        return result

    def handle_texts_before_after(self, normalized: str) -> str:
        """Handle text before and after the key placeholder."""
        if not self.text_before and not self.text_after:
            return normalized

        logger.debug(f"handle_texts_before_after: {normalized=}")
        # no need for further processing
        # (text_before="the ") but key: ("the {nat_en} actors") already in formatted_data_ci so no need to replace
        normalized_strip = normalized.strip()
        if self.formatted_data_ci.get(normalized_strip, ""):
            logger.debug(f"handle_texts_before_after: found directly {normalized_strip=} in formatted_data_ci")
            return normalized

        if self.text_before:
            if f"{self.text_before}{self.key_placeholder}" in normalized:
                normalized = normalized.replace(f"{self.text_before}{self.key_placeholder}", self.key_placeholder)

            # no need for further processing
            # (text_after=" people") but key: ("{nat_en} people actors") already in formatted_data_ci so no need to replace
            if self.formatted_data_ci.get(normalized_strip, ""):
                return normalized

        if self.text_after:
            if f"{self.key_placeholder}{self.text_after}" in normalized:
                normalized = normalized.replace(f"{self.key_placeholder}{self.text_after}", self.key_placeholder)

        return normalized

    @functools.lru_cache(maxsize=None)
    def normalize_category(self, category: str, sport_key: str) -> str:
        """Replace the matched sport key with the key placeholder."""
        # Normalize the category by removing extra spaces
        normalized_category = " ".join(category.split())

        normalized = re.sub(
            rf"(?<!{self.regex_filter}){re.escape(sport_key)}(?!{self.regex_filter})",
            f"{self.key_placeholder}",
            f" {normalized_category.strip()} ",
            flags=re.IGNORECASE,
            count=1,
        )

        normalized = self.handle_texts_before_after(normalized)
        return normalized.strip()

    def normalize_category_with_key(self, category: str) -> tuple[str, str]:
        """
        Normalize nationality placeholders within a category string.

        Example:
            normal:
                category:"Yemeni national football teams", result: "natar national football teams"
            model_data_double:
                category='{nat_en} action drama films', key='action drama', result='{nat_en} {film_key} films'
        """
        key = self.match_key(category)
        result = ""
        if key:
            result = self.normalize_category(category, key)
            logger.debug(f">>> normalize_category_with_key: {category=}, {key=}, {result=}")

        return key, result

    def get_template(self, sport_key: str, category: str) -> str:
        """Lookup template in a case-insensitive dict."""
        normalized = self.normalize_category(category, sport_key)
        logger.debug(f"[] get_template:  : {normalized=}")
        # Case-insensitive key lookup
        return self.formatted_data_ci.get(normalized.lower(), "")

    def get_template_ar(self, template_key: str) -> str:
        """Lookup template in a case-insensitive dict."""
        # Case-insensitive key lookup
        template_key = template_key.lower()
        logger.debug(f"get_template_ar: {template_key=}")
        result = self.formatted_data_ci.get(template_key, "")

        if not result:
            if template_key.startswith("category:"):
                template_key = template_key.replace("category:", "")
                result = self.formatted_data_ci.get(template_key, "")
            else:
                result = self.formatted_data_ci.get(f"category:{template_key}", "")

        logger.debug(f"get_template_ar: {template_key=}, {result=}")
        return result

    def get_key_label(self, sport_key: str) -> Any:
        """Return the Arabic label mapped to the provided key if present."""
        return self.data_list_ci.get(sport_key)

    def _search(self, category: str) -> str:
        """End-to-end resolution."""
        logger.debug("$$$ start _search(): ")
        logger.debug(f"++++++++ _search {self.__class__.__name__} ++++++++ ")

        if self.formatted_data_ci.get(category):
            return self.formatted_data_ci[category]

        sport_key = self.match_key(category)

        if not sport_key:
            logger.debug(f"No sport key matched for {category=}")
            return ""

        sport_label = self.get_key_label(sport_key)
        if not sport_label:
            logger.debug(f'No sport label matched for sport key: "{sport_key}"')
            return ""

        template_label = self.get_template(sport_key, category)
        if not template_label:
            logger.debug(f'No template label matched for sport key: "{sport_key}" and {category=}')
            return ""

        result = self.apply_pattern_replacement(template_label, sport_label)
        logger.debug(f"[] apply_pattern_replacement: {result=}")

        logger.debug(f"++++++++ end {self.__class__.__name__} ++++++++ ")

        return result

    def apply_pattern_replacement(self, template_label: str, sport_label: Any) -> str:
        """Replace value placeholder once template is chosen. Override in subclasses."""
        raise NotImplementedError("Subclasses must implement apply_pattern_replacement")

    def replace_value_placeholder(self, label: str, value: Any) -> str:
        """Replace placeholder. Override in subclasses if needed."""
        raise NotImplementedError("Subclasses must implement replace_value_placeholder")

    @functools.lru_cache(maxsize=None)
    def search(self, category: str) -> str:
        """Public wrapper around ``_search`` with caching."""
        return self._search(category)

    @functools.lru_cache(maxsize=None)
    def create_label(self, category: str) -> str:
        """Public wrapper around ``_search`` with caching."""
        return self._search(category)

    @functools.lru_cache(maxsize=None)
    def search_all(self, category: str) -> str:
        """Public wrapper around ``_search`` with caching."""
        return self._search(category)

    def check_placeholders(self, category: str, result: str) -> str:
        if "{" in result:
            logger.warning(f">>> search_all_category Found unprocessed placeholders in {category=}: {result=}")
            return ""
        return result

    def search_all_category(self, category: str) -> str:
        logger.debug("--" * 5)
        logger.debug(">> search_all_category start")
        normalized_category = category.lower().replace("category:", "")

        result = self._search(normalized_category)

        if result and category.lower().startswith("category:"):
            result = "تصنيف:" + result

        result = self.check_placeholders(category, result)

        logger.debug(">> search_all_category end")
        return result



#!/usr/bin/python3
"""
Extended tests for FormatDataDouble class.

This module provides exhaustive tests for:
1. splitter parameter
2. sort_ar_labels parameter
"""

import pytest

from ArWikiCats.translations_formats import FormatDataDouble


@pytest.fixture
def base_data():
    formatted_data = {
        "{film_key} films": "أفلام {film_ar}",
        "{film_key} movies": "أفلام {film_ar}",
    }
    data_list = {
        "action": "أكشن",
        "drama": "دراما",
        "comedy": "كوميدي",
        "horror": "رعب",
    }
    return formatted_data, data_list


def test_splitter_default_space(base_data):
    formatted_data, data_list = base_data
    bot = FormatDataDouble(
        formatted_data=formatted_data,
        data_list=data_list,
        key_placeholder="{film_key}",
        value_placeholder="{film_ar}",
    )
    result = bot.search("action drama films")
    assert result == "أفلام أكشن دراما"


def test_splitter_underscore(base_data):
    formatted_data, data_list = base_data
    bot = FormatDataDouble(
        formatted_data=formatted_data,
        data_list=data_list,
        key_placeholder="{film_key}",
        value_placeholder="{film_ar}",
        splitter="[_ ]",
    )
    assert bot.search("action_drama films") == "أفلام أكشن دراما"
    # Also matches with space since "[_ ]" is a character class matching both
    assert bot.search("action drama films") == "أفلام أكشن دراما"


def test_sort_simple_no_sort(base_data):
    formatted_data, data_list = base_data
    bot = FormatDataDouble(
        formatted_data=formatted_data,
        data_list=data_list,
        key_placeholder="{film_key}",
        value_placeholder="{film_ar}",
        sort_ar_labels=False,
        log_multi_cache=False,
    )
    result = bot.search("action drama films")
    assert result == "أفلام أكشن دراما"

    result2 = bot.search("drama action movies")
    assert result2 == "أفلام دراما أكشن"


def test_sort_simple_with_sort(base_data):
    formatted_data, data_list = base_data
    bot = FormatDataDouble(
        formatted_data=formatted_data,
        data_list=data_list,
        key_placeholder="{film_key}",
        value_placeholder="{film_ar}",
        sort_ar_labels=True,
    )
    # أكشن (Action) < دراما (Drama)
    assert bot.search("action drama films") == "أفلام أكشن دراما"
    assert bot.search("drama action films") == "أفلام أكشن دراما"


def test_sort_alphabetical_complex(base_data):
    formatted_data, data_list = base_data
    bot = FormatDataDouble(
        formatted_data=formatted_data,
        data_list=data_list,
        key_placeholder="{film_key}",
        value_placeholder="{film_ar}",
        sort_ar_labels=True,
    )
    # كوميدي (Comedy) Kaf > دراما (Drama) Dal ?
    # Let's check: د is U+062F, ك is U+0643. So د < ك.
    # So "دراما كوميدي"
    assert bot.search("comedy drama films") == "أفلام دراما كوميدي"


def test_splitter_regex_chars(base_data):
    formatted_data, data_list = base_data
    # Use something that needs escaping if literal, but code doesn't escape it.
    # '.' matches any char.
    bot = FormatDataDouble(
        formatted_data=formatted_data,
        data_list=data_list,
        key_placeholder="{film_key}",
        value_placeholder="{film_ar}",
        splitter=".",
    )
    # If not escaped, 'action.drama' matches 'actionXdrama'
    assert bot.search("action.drama films") == "أفلام أكشن دراما"
    # This might pass even if not escaped because '.' matches '.'.
    # But does it match 'action_drama'?
    # If not escaped, yes!
    assert bot.search("action_drama films") == "أفلام أكشن دراما"


def test_ar_joiner_default(base_data):
    formatted_data, data_list = base_data
    bot = FormatDataDouble(
        formatted_data=formatted_data,
        data_list=data_list,
        key_placeholder="{film_key}",
        value_placeholder="{film_ar}",
    )
    result = bot.search("action drama films")
    assert result == "أفلام أكشن دراما"


def test_ar_joiner_and(base_data):
    formatted_data, data_list = base_data
    bot = FormatDataDouble(
        formatted_data=formatted_data,
        data_list=data_list,
        key_placeholder="{film_key}",
        value_placeholder="{film_ar}",
        ar_joiner=" و ",
    )
    # "أكشن" + " و " + "دراما" -> "أفلام أكشن و دراما"
    result = bot.search("action drama films")
    assert result == "أفلام أكشن و دراما"


def test_ar_joiner_dash(base_data):
    formatted_data, data_list = base_data
    bot = FormatDataDouble(
        formatted_data=formatted_data,
        data_list=data_list,
        key_placeholder="{film_key}",
        value_placeholder="{film_ar}",
        ar_joiner="-",
    )
    # "أكشن" + "-" + "دراما" -> "أفلام أكشن-دراما"
    result = bot.search("action drama films")
    assert result == "أفلام أكشن-دراما"


def test_ar_joiner_with_sort(base_data):
    formatted_data, data_list = base_data
    bot = FormatDataDouble(
        formatted_data=formatted_data,
        data_list=data_list,
        key_placeholder="{film_key}",
        value_placeholder="{film_ar}",
        sort_ar_labels=True,
        ar_joiner=" و ",
    )
    # أكشن (Action) < دراما (Drama)
    # "action drama" -> "أفلام أكشن و دراما"
    assert bot.search("action drama films") == "أفلام أكشن و دراما"
    # "drama action" -> "أفلام أكشن و دراما"
    assert bot.search("drama action films") == "أفلام أكشن و دراما"


def test_ar_joiner_none(base_data):
    formatted_data, data_list = base_data
    # ar_joiner=None should default to space " "
    bot = FormatDataDouble(
        formatted_data=formatted_data,
        data_list=data_list,
        key_placeholder="{film_key}",
        value_placeholder="{film_ar}",
        ar_joiner=None,
    )
    result = bot.search("action drama films")
    assert result == "أفلام أكشن دراما"


def test_ar_joiner_empty(base_data):
    formatted_data, data_list = base_data
    # ar_joiner="" should ALSO default to space " " because of 'or " "' in __init__
    bot = FormatDataDouble(
        formatted_data=formatted_data,
        data_list=data_list,
        key_placeholder="{film_key}",
        value_placeholder="{film_ar}",
        ar_joiner="",
    )
    result = bot.search("action drama films")
    assert result == "أفلام أكشن دراما"



"""
jobs data
"""

from __future__ import annotations

from typing import Dict, Iterable, List, Mapping

from ...helps import len_print
from .jobs_defs import GenderedLabel, GenderedLabelMap, combine_gender_labels


def _build_religious_job_labels(
    religions: Mapping[str, GenderedLabel],
    roles: Mapping[str, GenderedLabel],
) -> GenderedLabelMap:
    """Generate gendered labels for religious roles.

    Args:
        religions: Mapping of religion identifiers to their gendered labels.
        roles: Mapping of religious roles to gendered labels.

    Returns:
        A dictionary keyed by string templates representing the combination of
        religion and role, matching the original dataset used by downstream
        modules.
    """

    combined_roles: GenderedLabelMap = {}
    for religion_key, religion_labels in religions.items():
        if not religion_key or not religion_labels:
            continue
        for role_key, role_labels in roles.items():
            if not role_key or not role_labels:
                continue
            females_label = combine_gender_labels(role_labels["females"], religion_labels["females"])
            males_label = combine_gender_labels(role_labels["males"], religion_labels["males"])

            if males_label or females_label:
                combined_roles[f"{religion_key} {role_key}"] = {
                    "males": males_label,
                    "females": females_label,
                }

    return combined_roles


def _build_painter_job_labels(
    painter_styles: Mapping[str, GenderedLabel],
    painter_roles: Mapping[str, GenderedLabel],
    painter_categories: Mapping[str, str],
) -> GenderedLabelMap:
    """Construct gendered labels for painting and artistic roles.

    Args:
        painter_styles: Mapping of painter descriptors (e.g. ``symbolist``) to
            their gendered Arabic forms.
        painter_roles: Mapping of artistic roles associated with painting.
        painter_categories: Additional label categories that are appended as
            human-readable Arabic strings.

    Returns:
        A dictionary containing both base roles and combined painter role
        variants.
    """
    # _build_painter_job_labels(PAINTER_STYLES, PAINTER_ROLE_LABELS, PAINTER_CATEGORY_LABELS)
    combined_data: GenderedLabelMap = {role_key: role_labels for role_key, role_labels in painter_roles.items()}

    combined_data.update({_style: _labels for _style, _labels in painter_styles.items() if _style != "history"})
    for style_key, style_labels in painter_styles.items():
        for role_key, role_labels in painter_roles.items():
            composite_key = f"{style_key} {role_key}"

            males_label = combine_gender_labels(role_labels["males"], style_labels["males"])
            females_label = combine_gender_labels(role_labels["females"], style_labels["females"])

            combined_data[composite_key] = {
                "males": males_label,
                "females": females_label,
            }
    for painter_category, category_label in painter_categories.items():
        if not painter_category or not category_label:
            continue
        combined_data[f"{painter_category} painters"] = {
            "males": f"رسامو {category_label}",
            "females": f"رسامات {category_label}",
        }
        combined_data[f"{painter_category} artists"] = {
            "males": f"فنانو {category_label}",
            "females": f"فنانات {category_label}",
        }

    return combined_data


def _build_military_job_labels(
    military_prefixes: Mapping[str, GenderedLabel],
    military_roles: Mapping[str, GenderedLabel],
    excluded_prefixes: Iterable[str],
) -> GenderedLabelMap:
    """Construct gendered labels for military related jobs.

    Args:
        military_prefixes: Base labels that modify the general military roles.
        military_roles: Roles that can be combined with each prefix.
        excluded_prefixes: Prefix keys that should not be added directly to the
            result set but are still used for composite roles.

    Returns:
        A dictionary of gendered labels covering both base roles and composite
        role names.
    """
    excluded = set(excluded_prefixes)

    combined_roles: GenderedLabelMap = {role_key: role_labels for role_key, role_labels in military_roles.items()}

    combined_roles.update(
        {
            prefix_key: prefix_labels
            for prefix_key, prefix_labels in military_prefixes.items()
            if prefix_key not in excluded
        }
    )

    for military_key, prefix_labels in military_prefixes.items():
        for role_key, role_labels in military_roles.items():
            composite_key = f"{military_key} {role_key}"
            males_label = combine_gender_labels(role_labels["males"], prefix_labels["males"])
            females_label = combine_gender_labels(role_labels["females"], prefix_labels["females"])
            combined_roles[composite_key] = {
                "males": males_label,
                "females": females_label,
            }

    return combined_roles


# --- Religious role definitions -------------------------------------------------
# (?<!\w)(shi'a\ muslims|sunni\ muslims|shia\ muslims|shi'a\ muslim|sunni\ muslim|shia\ muslim|episcopalians|evangelical|christians|protestant|anglicans|christian|methodist|religious|venerated|anglican|buddhist|bahá'ís|yazidis|islamic|muslims|muslim|coptic|hindus|jewish|zaydis|saints|hindu|zaydi|sufis|nazi|jews)(?!\w)
RELIGIOUS_KEYS_PP: GenderedLabelMap = {
    "bahá'ís": {"males": "بهائيون", "females": "بهائيات"},
    "baháís": {"males": "بهائيون", "females": "بهائيات"},
    "yazidis": {"males": "يزيديون", "females": "يزيديات"},
    "christians": {"males": "مسيحيون", "females": "مسيحيات"},
    "anglican": {"males": "أنجليكيون", "females": "أنجليكيات"},
    "anglicans": {"males": "أنجليكيون", "females": "أنجليكيات"},
    "episcopalians": {"males": "أسقفيون", "females": "أسقفيات"},
    "christian": {"males": "مسيحيون", "females": "مسيحيات"},
    "buddhist": {"males": "بوذيون", "females": "بوذيات"},
    "nazi": {"males": "نازيون", "females": "نازيات"},
    "muslim": {"males": "مسلمون", "females": "مسلمات"},
    "coptic": {"males": "أقباط", "females": "قبطيات"},
    "islamic": {"males": "إسلاميون", "females": "إسلاميات"},
    "hindus": {"males": "هندوس", "females": "هندوسيات"},
    "hindu": {"males": "هندوس", "females": "هندوسيات"},
    "protestant": {"males": "بروتستانتيون", "females": "بروتستانتيات"},
    "methodist": {"males": "ميثوديون لاهوتيون", "females": "ميثوديات لاهوتيات"},
    "jewish": {"males": "يهود", "females": "يهوديات"},
    "jews": {"males": "يهود", "females": "يهوديات"},
    "zaydis": {"males": "زيود", "females": "زيديات"},
    "zaydi": {"males": "زيود", "females": "زيديات"},
    "sufis": {"males": "صوفيون", "females": "صوفيات"},
    "religious": {"males": "دينيون", "females": "دينيات"},
    "muslims": {"males": "مسلمون", "females": "مسلمات"},
    "shia muslims": {"males": "مسلمون شيعة", "females": "مسلمات شيعيات"},
    "shi'a muslims": {"males": "مسلمون شيعة", "females": "مسلمات شيعيات"},
    "sunni muslims": {"males": "مسلمون سنة", "females": "مسلمات سنيات"},
    "shia muslim": {"males": "مسلمون شيعة", "females": "مسلمات شيعيات"},
    "shi'a muslim": {"males": "مسلمون شيعة", "females": "مسلمات شيعيات"},
    "sunni muslim": {"males": "مسلمون سنة", "females": "مسلمات سنيات"},
    "evangelical": {"males": "إنجيليون", "females": "إنجيليات"},
    "venerated": {"males": "مبجلون", "females": "مبجلات"},
    "saints": {"males": "قديسون", "females": "قديسات"},
}

NAT_BEFORE_OCC_BASE: List[str] = [
    "murdered abroad",
    "contemporary",
    "tour de france stage winners",
    "deafblind",
    "deaf",
    "blind",
    "jews",
    "women's rights activists",
    "female rights activists",
    "human rights activists",
    "imprisoned",
    "imprisoned abroad",
    "conservationists",
    "expatriate",
    "defectors",
    "scholars of islam",
    "scholars-of-islam",
    "amputees",
    "expatriates",
    "executed abroad",
    "emigrants",
]

NAT_BEFORE_OCC = list(NAT_BEFORE_OCC_BASE)
NAT_BEFORE_OCC.extend(key for key in RELIGIOUS_KEYS_PP.keys())

RELIGIOUS_ROLE_LABELS: GenderedLabelMap = {
    "christians": {"males": "مسيحيون", "females": "مسيحيات"},
    "venerated": {"males": "مبجلون", "females": "مبجلات"},
    "missionaries": {"males": "مبشرون", "females": "مبشرات"},
    "evangelical": {"males": "إنجيليون", "females": "إنجيليات"},
    "monks": {"males": "رهبان", "females": "راهبات"},
    "nuns": {"males": "", "females": "راهبات"},
    "saints": {"males": "قديسون", "females": "قديسات"},
    "astrologers": {"males": "منجمون", "females": "منجمات"},
    "leaders": {"males": "قادة", "females": "قائدات"},
    "bishops": {"males": "أساقفة", "females": ""},
    "actors": {"males": "ممثلون", "females": "ممثلات"},
    "theologians": {"males": "لاهوتيون", "females": "لاهوتيات"},
    "clergy": {"males": "رجال دين", "females": "سيدات دين"},
    "religious leaders": {"males": "قادة دينيون", "females": "قائدات دينيات"},
}


# --- Painter role definitions ---------------------------------------------------
PAINTER_STYLES: GenderedLabelMap = {
    "symbolist": {"males": "رمزيون", "females": "رمزيات"},
    "history": {"males": "تاريخيون", "females": "تاريخيات"},
    "romantic": {"males": "رومانسيون", "females": "رومانسيات"},
    "neoclassical": {"males": "كلاسيكيون حديثون", "females": "كلاسيكيات حديثات"},
    "religious": {"males": "دينيون", "females": "دينيات"},
}

PAINTER_ROLE_LABELS: GenderedLabelMap = {
    "painters": {"males": "رسامون", "females": "رسامات"},
    "artists": {"males": "فنانون", "females": "فنانات"},
}

PAINTER_CATEGORY_LABELS: Dict[str, str] = {
    "make-up": "مكياج",
    "comics": "قصص مصورة",
    "marvel comics": "مارفال كومكس",
    "manga": "مانغا",
    "landscape": "مناظر طبيعية",
    "wildlife": "حياة برية",
    "portrait": "بورتريه",
    "animal": "حيوانات",
    "genre": "نوع",
    "still life": "طبيعة صامتة",
}

# --- Military role definitions --------------------------------------------------
MILITARY_PREFIXES: GenderedLabelMap = {
    "military": {"males": "عسكريون", "females": "عسكريات"},
    "politicians": {"males": "سياسيون", "females": "سياسيات"},
    "nazi": {"males": "نازيون", "females": "نازيات"},
    "literary": {"males": "أدبيون", "females": "أدبيات"},
    "organizational": {"males": "تنظيميون", "females": "تنظيميات"},
}

MILITARY_ROLE_LABELS: GenderedLabelMap = {
    "theorists": {"males": "منظرون", "females": "منظرات"},
    "musicians": {"males": "موسيقيون", "females": "موسيقيات"},
    "engineers": {"males": "مهندسون", "females": "مهندسات"},
    "leaders": {"males": "قادة", "females": "قائدات"},
    "officers": {"males": "ضباط", "females": "ضابطات"},
    "historians": {"males": "مؤرخون", "females": "مؤرخات"},
    "strategists": {"males": "استراتيجيون", "females": "استراتيجيات"},
    "nurses": {"males": "ممرضون", "females": "ممرضات"},
}

EXCLUDED_MILITARY_PREFIXES = ("military", "literary")


# --- Aggregate outputs ----------------------------------------------------------
MEN_WOMENS_JOBS_2: GenderedLabelMap = {}
MEN_WOMENS_JOBS_2.update(_build_religious_job_labels(RELIGIOUS_KEYS_PP, RELIGIOUS_ROLE_LABELS))

MEN_WOMENS_JOBS_2.update(_build_painter_job_labels(PAINTER_STYLES, PAINTER_ROLE_LABELS, PAINTER_CATEGORY_LABELS))

MEN_WOMENS_JOBS_2.update(
    _build_military_job_labels(
        MILITARY_PREFIXES,
        MILITARY_ROLE_LABELS,
        EXCLUDED_MILITARY_PREFIXES,
    )
)

__all__ = [
    "MEN_WOMENS_JOBS_2",
    "MILITARY_PREFIXES",
    "MILITARY_ROLE_LABELS",
    "PAINTER_CATEGORY_LABELS",
    "PAINTER_ROLE_LABELS",
    "PAINTER_STYLES",
    "RELIGIOUS_KEYS_PP",
    "NAT_BEFORE_OCC",
]

len_print.data_len(
    "jobs_data_basic.py",
    {
        "MEN_WOMENS_JOBS_2": MEN_WOMENS_JOBS_2,
        "MILITARY_PREFIXES": MILITARY_PREFIXES,
        "MILITARY_ROLE_LABELS": MILITARY_ROLE_LABELS,
        "PAINTER_CATEGORY_LABELS": PAINTER_CATEGORY_LABELS,
        "PAINTER_ROLE_LABELS": PAINTER_ROLE_LABELS,
        "PAINTER_STYLES": PAINTER_STYLES,
        "RELIGIOUS_KEYS_PP": RELIGIOUS_KEYS_PP,
        "NAT_BEFORE_OCC": NAT_BEFORE_OCC,
    },
)



"""
Tests
"""

import re

import pytest
from load_one_data import dump_diff, one_dump_test

from ArWikiCats.genders_resolvers.nat_genders_pattern_multi import genders_sports_resolver

test_sport_bot_data = {
    "footballers": "لاعبو ولاعبات كرة قدم",
    "mens footballers": "لاعبو كرة قدم",
    "softball players": "لاعبو ولاعبات كرة لينة",
    "male beach soccer players": "لاعبو كرة قدم شاطئية",
    "male squash players": "لاعبو اسكواش",
    "male submission wrestling players": "لاعبو مصارعة خضوع",
    "male sumo players": "لاعبو سومو",
    "male surfing players": "لاعبو ركمجة",
    "male swimming players": "لاعبو سباحة",
    "male synchronised swimming players": "لاعبو سباحة متزامنة",
    "male synchronized swimming players": "لاعبو سباحة متزامنة",
    "female players of american-football": "لاعبات كرة قدم أمريكية",
    "male players of american-football": "لاعبو كرة قدم أمريكية",
    "players of american-football": "لاعبو ولاعبات كرة قدم أمريكية",
    "yemeni players of american-football": "لاعبو ولاعبات كرة قدم أمريكية يمنيون",
    "yemeni male players of american-football": "لاعبو كرة قدم أمريكية يمنيون",
    "yemeni female players of american-football": "لاعبات كرة قدم أمريكية يمنيات",
}


@pytest.mark.parametrize("category, expected", test_sport_bot_data.items(), ids=test_sport_bot_data.keys())
@pytest.mark.fast
def test_sport_bot(category: str, expected: str) -> None:
    label = genders_sports_resolver(category)
    assert label == expected



"""
Tests
"""

import pytest

from ArWikiCats.legacy_bots.ma_bots2.reg_result import get_reg_result


def test_get_reg_result_1() -> None:
    # Test with basic inputs
    result = get_reg_result("Category:19th government of turkey")
    assert result.year_at_first_strip == "19th"
    assert result.In == ""
    assert result.country == "government of turkey"
    assert result.cat_test == "government of turkey"


def test_get_reg_result() -> None:
    # Test with basic inputs
    result = get_reg_result("Category:2025 in fishes")
    assert hasattr(result, "year_at_first")
    assert result.year_at_first_strip == "2025"
    assert hasattr(result, "In")
    assert hasattr(result, "country")
    assert hasattr(result, "cat_test")

    # Test with different parameters
    result_various = get_reg_result("category:year in type")
    assert hasattr(result_various, "year_at_first")
    assert hasattr(result_various, "In")
    assert hasattr(result_various, "country")
    assert hasattr(result_various, "cat_test")


class TestYearExtraction:
    @pytest.mark.parametrize(
        "category,expected",
        [
            # Basic year
            ("Category:1999 events in France", "1999"),
            ("Category:2020 births", "2020"),
            # Year range
            ("Category:1933–83 American Soccer League", "1933–83"),
            ("Category:1933-83 American Soccer League", "1933-83"),
            ("Category:1933−83 American Soccer League", "1933−83"),
            # Decade with s
            ("Category:1990s in music", "1990s"),
            # No year → should be empty
            ("Category:Animals of North America", ""),
            ("Category:Sports in Europe", ""),
            # Month test (month should remain ignored)
            ("Category:January 1999 events", "January 1999"),
            ("Category:February 2021 disasters", "february 2021"),
        ],
    )
    def test_year(self, category: str, expected: str) -> None:
        out = get_reg_result(category)
        assert out.year_at_first_strip.lower() == expected.lower()

    @pytest.mark.parametrize(
        "category,expected",
        [
            # BCE/BC (centuries)
            ("Category:2nd century BC", "2nd century BC"),
            ("Category:5th century BCE", "5th century BCE"),
            ("Category:1st millennium BC", "1st millennium BC"),
            # Plain century
            ("Category:20th century", "20th century"),
            # Decade with s
            ("Category:10s BC", "10s BC"),
        ],
    )
    def test_year2(self, category: str, expected: str) -> None:
        out = get_reg_result(category)
        assert out.year_at_first.lower() == expected.lower()


# -----------------------------------------------------------
# 6) Tests for cat_test modification after removing year
# -----------------------------------------------------------
@pytest.mark.fast
class TestCatTestModification:
    def test_cat_test_year_removed(self) -> None:
        category = "Category:1999 births in France"
        out = get_reg_result(category)
        assert "1999" not in out.cat_test

    def test_cat_test_unchanged_if_no_year(self) -> None:
        category = "Category:births in France"
        out = get_reg_result(category)
        assert out.cat_test == "births in france"


# -----------------------------------------------------------
# 7) Tests for month suppression (tita_year_no_month)
# -----------------------------------------------------------
@pytest.mark.fast
class TestMonthSuppression:
    @pytest.mark.parametrize(
        "category,expected",
        [
            ("Category:January 1999 events", "january 1999"),
            ("Category:December 2020 births", "december 2020"),
        ],
    )
    def test_month_suppression(self, category: str, expected: str) -> None:
        out = get_reg_result(category)
        assert out.year_at_first_strip == expected


# -----------------------------------------------------------
# 8) Tests for BCE / BC variations
# -----------------------------------------------------------


class TestBCE_BC:
    @pytest.mark.parametrize(
        "category,expected",
        [
            ("Category:10th century BC", "10th century BC"),
            ("Category:5th century BCE", "5th century BCE"),
            ("Category:1st millennium BC", "1st millennium BC"),
            ("Category:2nd millennium BCE", "2nd millennium BCE"),
        ],
    )
    def test_bce(self, category: str, expected: str) -> None:
        out = get_reg_result(category)
        assert out.year_at_first.lower() == expected.lower()


# -----------------------------------------------------------
# 11) Edge cases
# -----------------------------------------------------------
@pytest.mark.fast
class TestEdgeCases:
    def test_empty_category(self) -> None:
        out = get_reg_result("")
        assert out.year_at_first == ""
        assert out.In == ""
        assert out.country == ""

    def test_only_category_prefix(self) -> None:
        cat = "Category:"
        out = get_reg_result(cat)
        assert out.year_at_first == ""

    def test_spaces_only(self) -> None:
        cat = "Category:     "
        out = get_reg_result(cat)
        assert out.year_at_first == ""



#!/usr/bin/python3
"""
Module for double-key category translation formatting.

This module provides the FormatDataDouble class for translating category strings
that contain two dynamic elements that need to be matched and combined. It is used
for categories like "action drama films" where both "action" and "drama" are
separate keys that need to be identified and their labels combined.

Classes:
    FormatDataDouble: Handles double-key template-driven category translations.

Example:
    >>> from ArWikiCats.translations_formats.DataModel import FormatDataDouble
    >>> formatted_data = {
    ...     "{film_key} films": "أفلام {film_label}",
    ... }
    >>> data_list = {
    ...     "action": "أكشن",
    ...     "drama": "دراما",
    ...     "comedy": "كوميدي",
    ... }
    >>> bot = FormatDataDouble(formatted_data, data_list, key_placeholder="{film_key}", value_placeholder="{film_label}")
    >>> bot.search("action drama films")
    'أفلام أكشن دراما'
"""

import functools
import re
from typing import Dict, Optional

from ..DataModel.model_data_base import FormatDataBase
from ..formats_logger import logger


class FormatDataDouble(FormatDataBase):
    """
    Handles double-key template-driven category translations.

    This class extends FormatDataBase to handle categories where two adjacent
    keys from the data_list appear together (e.g., "action drama films").
    It can match both single keys and pairs of keys, combining their Arabic
    labels in the correct order.

    Attributes:
        formatted_data (Dict[str, str]): Template patterns mapping English patterns to Arabic templates.
        data_list (Dict[str, str]): Key-to-Arabic-label mappings for replacements.
        key_placeholder (str): Placeholder used in formatted_data keys.
        value_placeholder (str): Placeholder used in formatted_data values.
        text_after (str): Text to append after the translated label.
        text_before (str): Text to prepend before the translated label.
        splitter (str): Separator used between keys in input strings.
        ar_joiner (str): Separator used between Arabic labels in output.
        sort_ar_labels (bool): Whether to sort Arabic labels alphabetically.
        alternation (str): Regex alternation string for keys.
        pattern (re.Pattern): Regex pattern for single key matching.
        pattern_double (re.Pattern): Regex pattern for matching two adjacent keys.
        keys_to_split (dict): Cache mapping combined keys to their component parts.
        put_label_last (dict): Keys whose labels should appear last in combinations.
        search_multi_cache (dict): Cache for combined label lookups.

    Example:
        >>> data_list = {
        ...     "action": "أكشن",
        ...     "drama": "دراما",
        ... }
        >>> bot = FormatDataDouble(
        ...     formatted_data={"{genre} films": "أفلام {genre_label}"},
        ...     data_list=data_list,
        ...     key_placeholder="{genre}",
        ...     value_placeholder="{genre_label}",
        ... )
        >>> bot.search("action drama films")
        'أفلام أكشن دراما'
    """

    def __init__(
        self,
        formatted_data: Dict[str, str],
        data_list: Dict[str, str],
        key_placeholder: str = "xoxo",
        value_placeholder: str = "xoxo",
        text_after: str = "",
        text_before: str = "",
        splitter: str = " ",
        ar_joiner: str = " ",
        sort_ar_labels: bool = False,
        log_multi_cache: bool = True,
    ):
        """Prepare helpers for matching and formatting template-driven labels."""
        super().__init__(
            formatted_data=formatted_data,
            data_list=data_list,
            key_placeholder=key_placeholder,
            text_after=text_after,
            text_before=text_before,
        )
        self.log_multi_cache = log_multi_cache
        self.sort_ar_labels = sort_ar_labels
        self.value_placeholder = value_placeholder
        self.keys_to_split = {}
        self.put_label_last = {}
        self.search_multi_cache = {}
        self.splitter = splitter or " "
        self.ar_joiner = ar_joiner or " "

        self.alternation: str = self.create_alternation()
        self.pattern = self.keys_to_pattern()

        self.pattern_double = self.keys_to_pattern_double()

    def update_put_label_last(self, data: list[str] | set[str]) -> None:
        self.put_label_last = data

    def keys_to_pattern_double(self) -> Optional[re.Pattern[str]]:
        """
        Build a case-insensitive regex over lowercased keys of data_list.
        """
        if not self.data_list_ci:
            return None

        if self.alternation is None:
            self.alternation = self.create_alternation()

        data_pattern = rf"(?<!\w)({self.alternation})({self.splitter})({self.alternation})(?!\w)"
        return re.compile(data_pattern, re.I)

    @functools.lru_cache(maxsize=None)
    def match_key(self, category: str) -> str:
        """Return canonical lowercased key from data_list if found; else empty."""
        if not self.pattern:
            return ""

        # Normalize the category by removing extra spaces
        normalized_category = " ".join(category.split())
        logger.debug(f">!> match_key: {normalized_category=}")

        # TODO: check this
        if self.data_list_ci.get(normalized_category.lower()):
            logger.debug(f">>!!>> match_key: found in data_list_ci {normalized_category=}")
            return normalized_category.lower()

        match = self.pattern_double.search(f" {normalized_category} ")
        if match:
            first_key = match.group(1).lower()
            splitter = match.group(2).lower()
            second_key = match.group(3).lower()
            result = f"{first_key}{splitter}{second_key}"

            logger.debug(f">!> match_key: {first_key=}, {second_key=}")
            logger.debug(f">!> match_key: {result=}")
            self.keys_to_split[result] = [first_key, second_key]
            return result

        match2 = self.pattern.search(f" {normalized_category} ")
        result = match2.group(1).lower() if match2 else ""
        logger.debug(f"==== match_key {result=}")

        return result

    @functools.lru_cache(maxsize=None)
    def apply_pattern_replacement(self, template_label: str, sport_label: str) -> str:
        """Replace value placeholder once template is chosen."""
        final_label = template_label.replace(self.value_placeholder, sport_label)

        if self.value_placeholder not in final_label:
            return final_label.strip()

        return ""

    @functools.lru_cache(maxsize=None)
    def create_label_from_keys(self, part1: str, part2: str):
        """
        if "upcoming" in self.put_label_last we using:
            "أفلام قادمة رعب يمنية instead of "أفلام رعب قادمة يمنية"
        """

        first_label = self.data_list_ci.get(part1)
        second_label = self.data_list_ci.get(part2)

        if not first_label or not second_label:
            return ""

        label = f"{first_label}{self.ar_joiner}{second_label}"

        if part1 in self.put_label_last and part2 not in self.put_label_last:
            label = f"{second_label}{self.ar_joiner}{first_label}"

        if self.sort_ar_labels:
            labels_sorted = sorted([first_label, second_label])
            label = self.ar_joiner.join(labels_sorted)
        if self.log_multi_cache:
            self.search_multi_cache[f"{part2} {part1}"] = label

        return label

    def get_key_label(self, sport_key: str) -> str:
        """
        Return the Arabic label mapped to the provided key if present.
        Example:
            sport_key="action", result="أكشن"
            sport_key="action drama", result="أكشن دراما"
        """
        result = self.data_list_ci.get(sport_key)
        if result:
            return result

        if self.search_multi_cache.get(sport_key.lower()):
            return self.search_multi_cache[sport_key.lower()]

        if sport_key in self.keys_to_split:
            part1, part2 = self.keys_to_split[sport_key]
            return self.create_label_from_keys(part1, part2)

        return ""

    def replace_value_placeholder(self, label: str, value: str) -> str:
        # Replace placeholder
        logger.debug(f"!!!! replace_value_placeholder: {self.value_placeholder=}, {label=}, {value=}")
        return label.replace(self.value_placeholder, value)



#!/usr/bin/python3
"""
Tests for MultiDataFormatterYearAndFrom2 class with translation_category_relations integration.

This module tests the new methods added to MultiDataFormatterYearAndFrom2:
- get_relation_word: Find relation words in categories
- resolve_relation_label: Append Arabic relation words to labels
- get_relation_mapping: Access the translation_category_relations dictionary

Tests follow existing project conventions and use pytest parametrize for data-driven testing.

TODO: use MultiDataFormatterYearAndFrom2 in workflows.
"""

import pytest

from ArWikiCats.format_bots.relation_mapping import translation_category_relations
from ArWikiCats.time_resolvers.time_to_arabic import convert_time_to_arabic, match_time_en_first
from ArWikiCats.translations_formats import FormatDataFrom, MultiDataFormatterYearAndFrom2


def get_label(text: str) -> str:
    """Mock label lookup function for testing."""
    data = {
        "writers from Hong Kong": "كتاب من هونغ كونغ",
        "writers from yemen": "كتاب من اليمن",
        "writers from Crown of Aragon": "كتاب من تاج أرغون",
        "writers gg yemen": "كتاب من اليمن",
        "people from germany": "أشخاص من ألمانيا",
        "buildings in france": "مباني في فرنسا",
    }
    return data.get(text.lower(), "")


@pytest.fixture
def multi_bot() -> MultiDataFormatterYearAndFrom2:
    """Create a MultiDataFormatterYearAndFrom2 instance for testing."""
    formatted_data = {
        "{year1} {country1}": "{country1} في {year1}",
    }
    country_bot = FormatDataFrom(
        formatted_data=formatted_data,
        key_placeholder="{country1}",
        value_placeholder="{country1}",
        search_callback=get_label,
        match_key_callback=lambda x: x.replace("{year1}", "").strip(),
    )
    year_bot = FormatDataFrom(
        formatted_data={},
        key_placeholder="{year1}",
        value_placeholder="{year1}",
        search_callback=convert_time_to_arabic,
        match_key_callback=match_time_en_first,
    )
    return MultiDataFormatterYearAndFrom2(
        country_bot=country_bot,
        year_bot=year_bot,
        category_relation_mapping=translation_category_relations,
        other_key_first=True,
    )


class TestGetRelationWord:
    """Tests for get_relation_word method."""

    # Test data: (category, expected_key, expected_arabic)
    test_data = [
        ("People from Germany", "from", "من"),
        ("Buildings in France", "in", "في"),
        ("Works published by Oxford", "published by", "نشرتها"),
        ("Films directed by Spielberg", "directed by", "أخرجها"),
        ("Ships launched in 1910", "launched in", "أطلقت في"),
        ("Cities established in 1750", "established in", "أسست في"),
        ("Books written by Dickens", "written by", "كتبها"),
        ("Items manufactured in China", "manufactured in", "صنعت في"),
        ("Treaties concluded in Vienna", "concluded in", "أبرمت في"),
    ]

    @pytest.mark.parametrize(
        "category,expected_key,expected_arabic",
        test_data,
        ids=[t[0] for t in test_data],
    )
    def test_get_relation_word(
        self, multi_bot: MultiDataFormatterYearAndFrom2, category: str, expected_key: str, expected_arabic: str
    ) -> None:
        """Test that get_relation_word correctly identifies relation words."""
        key, arabic = multi_bot.get_relation_word(category)
        assert key == expected_key
        assert arabic == expected_arabic

    def test_no_relation_word(self, multi_bot: MultiDataFormatterYearAndFrom2) -> None:
        """Test that get_relation_word returns empty tuple when no relation found."""
        key, arabic = multi_bot.get_relation_word("Random category without relation")
        assert key == ""
        assert arabic == ""

    def test_relation_word_requires_spaces(self, multi_bot: MultiDataFormatterYearAndFrom2) -> None:
        """Test that relation words must be surrounded by spaces."""
        # "builtin" should not match "built in"
        key, arabic = multi_bot.get_relation_word("Somethingbuiltin somewhere")
        assert key == ""
        assert arabic == ""


class TestResolveRelationLabel:
    """Tests for resolve_relation_label method."""

    # Test data: (category, base_label, expected_result)
    test_data = [
        ("Writers from Yemen", "كتاب", "كتاب من"),
        ("People in Germany", "أشخاص", "أشخاص في"),
        ("Buildings built in France", "مباني", "مباني بنيت في"),
        ("Films directed by Spielberg", "أفلام", "أفلام أخرجها"),
        ("Works published by Oxford", "أعمال", "أعمال نشرتها"),
    ]

    @pytest.mark.parametrize(
        "category,base_label,expected",
        test_data,
        ids=[t[0] for t in test_data],
    )
    def test_resolve_relation_label(
        self, multi_bot: MultiDataFormatterYearAndFrom2, category: str, base_label: str, expected: str
    ) -> None:
        """Test that resolve_relation_label correctly appends Arabic relation words."""
        result = multi_bot.resolve_relation_label(category, base_label)
        assert result == expected

    def test_empty_base_label(self, multi_bot: MultiDataFormatterYearAndFrom2) -> None:
        """Test that resolve_relation_label handles empty base_label."""
        result = multi_bot.resolve_relation_label("Writers from Yemen", "")
        assert result == ""

    def test_empty_category(self, multi_bot: MultiDataFormatterYearAndFrom2) -> None:
        """Test that resolve_relation_label handles empty category."""
        result = multi_bot.resolve_relation_label("", "كتاب")
        assert result == "كتاب"

    def test_no_relation_in_category(self, multi_bot: MultiDataFormatterYearAndFrom2) -> None:
        """Test that resolve_relation_label returns original label when no relation found."""
        result = multi_bot.resolve_relation_label("Random text", "كتاب")
        assert result == "كتاب"

    def test_avoid_duplicate_relation(self, multi_bot: MultiDataFormatterYearAndFrom2) -> None:
        """Test that resolve_relation_label avoids adding duplicate relation words."""
        # "من" is already at the end of the base_label
        result = multi_bot.resolve_relation_label("Writers from Yemen", "كتاب من")
        assert result == "كتاب من"

    def test_avoid_duplicate_relation_with_country(self, multi_bot: MultiDataFormatterYearAndFrom2) -> None:
        """Test that resolve_relation_label avoids duplicates when relation is in middle."""
        # "من" appears in the middle followed by country
        result = multi_bot.resolve_relation_label("Writers from Yemen", "كتاب من اليمن")
        assert result == "كتاب من اليمن"

    def test_no_false_positive_duplicate_detection(self, multi_bot: MultiDataFormatterYearAndFrom2) -> None:
        """Test that substring matches don't produce false positives."""
        # "من" is part of "من الكتاب" but not as a standalone relation word
        # The method should still add "من" because it's not at word boundaries
        result = multi_bot.resolve_relation_label("Writers from Yemen", "أشخاص")
        assert result == "أشخاص من"


class TestGetRelationMapping:
    """Tests for get_relation_mapping method."""

    def test_get_relation_mapping_contains_expected_keys(self, multi_bot: MultiDataFormatterYearAndFrom2) -> None:
        """Test that get_relation_mapping contains expected relation words."""
        mapping = multi_bot.get_relation_mapping()
        # Check some expected keys
        assert "from" in mapping
        assert "in" in mapping
        assert "by" in mapping
        assert "published by" in mapping
        assert "directed by" in mapping

    def test_get_relation_mapping_contains_expected_values(self, multi_bot: MultiDataFormatterYearAndFrom2) -> None:
        """Test that get_relation_mapping contains expected Arabic translations."""
        mapping = multi_bot.get_relation_mapping()
        assert mapping["from"] == "من"
        assert mapping["in"] == "في"
        assert mapping["by"] == "حسب"
        assert mapping["published by"] == "نشرتها"


class TestIntegrationWithExistingFunctionality:
    """Integration tests to ensure new methods work with existing functionality."""

    def test_create_label_still_works(self, multi_bot: MultiDataFormatterYearAndFrom2) -> None:
        """Test that create_label still works after adding new methods."""
        result = multi_bot.create_label("14th-century writers from yemen")
        assert result == "كتاب من اليمن في القرن 14"

    def test_search_all_still_works(self, multi_bot: MultiDataFormatterYearAndFrom2) -> None:
        """Test that search_all still works after adding new methods."""
        result = multi_bot.search_all("14th-century writers from yemen")
        assert result == "كتاب من اليمن في القرن 14"

    def test_normalize_both_new_still_works(self, multi_bot: MultiDataFormatterYearAndFrom2) -> None:
        """Test that normalize_both_new still works after adding new methods."""
        result = multi_bot.normalize_both_new("14th-century writers from yemen")
        assert result.nat_key == "writers from yemen"
        assert result.other_key == "14th-century"


class TestEdgeCases:
    """Edge case tests for robust handling."""

    def test_multiple_relation_words(self, multi_bot: MultiDataFormatterYearAndFrom2) -> None:
        """Test category with multiple relation words returns first match."""
        # "from" appears before "in" in translation_category_relations
        key, arabic = multi_bot.get_relation_word("People from Germany in Europe")
        # Should match the first relation word found in the mapping
        assert key in ["from", "in"]

    def test_case_sensitivity(self, multi_bot: MultiDataFormatterYearAndFrom2) -> None:
        """Test that relation word matching is case-sensitive."""
        # The current implementation is case-sensitive
        key, arabic = multi_bot.get_relation_word("People FROM Germany")
        # Should not match because "FROM" != "from"
        assert key == ""
        assert arabic == ""

    def test_special_characters_in_relation(self, multi_bot: MultiDataFormatterYearAndFrom2) -> None:
        """Test relation words with special characters like hyphens."""
        key, arabic = multi_bot.get_relation_word("Schools for-deaf in New York")
        assert key == "for-deaf"
        assert arabic == "للصم"


class TestFormatDataFromClass:
    """Tests for the FormatDataFrom class."""

    def test_format_data_from_init(self) -> None:
        """Test FormatDataFrom initialization."""
        bot = FormatDataFrom(
            formatted_data={"test": "اختبار"},
            key_placeholder="{key}",
            value_placeholder="{value}",
            search_callback=lambda x: x,
            match_key_callback=lambda x: x,
        )
        assert bot.key_placeholder == "{key}"
        assert bot.value_placeholder == "{value}"
        assert bot.formatted_data == {"test": "اختبار"}
        assert "test" in bot.formatted_data_ci

    def test_format_data_from_match_key(self) -> None:
        """Test FormatDataFrom.match_key method."""
        bot = FormatDataFrom(
            formatted_data={},
            key_placeholder="{key}",
            value_placeholder="{value}",
            search_callback=lambda x: x,
            match_key_callback=lambda x: x.upper(),
        )
        result = bot.match_key("test")
        assert result == "TEST"

    def test_format_data_from_search(self) -> None:
        """Test FormatDataFrom.search method."""
        bot = FormatDataFrom(
            formatted_data={},
            key_placeholder="{key}",
            value_placeholder="{value}",
            search_callback=lambda x: f"searched: {x}",
            match_key_callback=lambda x: x,
        )
        result = bot.search("test")
        assert result == "searched: test"

    def test_format_data_from_normalize_category(self) -> None:
        """Test FormatDataFrom.normalize_category method."""
        bot = FormatDataFrom(
            formatted_data={},
            key_placeholder="{placeholder}",
            value_placeholder="{value}",
            search_callback=lambda x: x,
            match_key_callback=lambda x: x,
        )
        result = bot.normalize_category("category with key here", "key")
        assert result == "category with {placeholder} here"

    def test_format_data_from_get_template_ar(self) -> None:
        """Test FormatDataFrom.get_template_ar method."""
        bot = FormatDataFrom(
            formatted_data={"{year1} {country1}": "{country1} في {year1}"},
            key_placeholder="{country1}",
            value_placeholder="{country1}",
            search_callback=lambda x: x,
            match_key_callback=lambda x: x,
        )
        result = bot.get_template_ar("{year1} {country1}")
        assert result == "{country1} في {year1}"

    def test_format_data_from_get_template_ar_case_insensitive(self) -> None:
        """Test FormatDataFrom.get_template_ar is case-insensitive."""
        bot = FormatDataFrom(
            formatted_data={"{Year1} {Country1}": "{country1} في {year1}"},
            key_placeholder="{country1}",
            value_placeholder="{country1}",
            search_callback=lambda x: x,
            match_key_callback=lambda x: x,
        )
        result = bot.get_template_ar("{year1} {country1}")
        assert result == "{country1} في {year1}"

    def test_format_data_from_fixing_callback(self) -> None:
        """Test FormatDataFrom with fixing_callback."""
        bot = FormatDataFrom(
            formatted_data={},
            key_placeholder="{key}",
            value_placeholder="{value}",
            search_callback=lambda x: x,
            match_key_callback=lambda x: x,
            fixing_callback=lambda x: x.strip().upper(),
        )
        result = bot.replace_value_placeholder("  test {value}  ", "input")
        assert result == "TEST INPUT"



#
import pytest
from load_one_data import dump_diff, one_dump_test

# from ArWikiCats import resolve_arabic_category_label
from ArWikiCats.new_resolvers.bys_new import resolve_by_labels

data0 = {
    "by year": "حسب السنة",
    "decade": "العقد",
    "election": "الانتخابات",
    "by non-profit organizations by country": "حسب المنظمات غير الربحية حسب البلد",
    "non-profit organizations by continent": "المنظمات غير الربحية حسب القارة",
    "non-profit organizations by country": "المنظمات غير الربحية حسب البلد",
    "non-profit organizations by type": "المنظمات غير الربحية حسب الفئة",
    "non-profit organizations": "المنظمات غير الربحية",
    "non-profit publishers": "ناشرون غير ربحيون",
    "organized crime": "بواسطة الجريمة المنظمة",
    "populated place": "المكان المأهول",
    "sport": "الرياضة",
}

data1 = {
    "by academic discipline": "حسب التخصص الأكاديمي",
    "by architectural style": "حسب الطراز المعماري",
    "by artist nationality": "حسب جنسية الفنان",
    "by athletic event": "حسب حدث ألعاب القوى",
    "by audience": "حسب الجمهور",
    "by autonomous community": "حسب الحكم الذاتي",
    "by branch": "حسب الفرع",
    "by century and nationality": "حسب القرن والجنسية",
    "by century and occupation": "حسب القرن والمهنة",
    "by century of disestablishment": "حسب قرن الانحلال",
    "by century of establishment": "حسب قرن التأسيس",
    "by century of formal description": "حسب قرن الوصف",
    "by century": "حسب القرن",
    "by city of location": "حسب مدينة الموقع",
    "by city of setting": "حسب مدينة الأحداث",
    "by city": "حسب المدينة",
    "by club or team": "حسب النادي أو الفريق",
    "by club": "حسب النادي",
    "by competition": "حسب المنافسة",
    "by conflict": "حسب النزاع",
    "by continent of setting": "حسب قارة الأحداث",
    "by continent": "حسب القارة",
    "by country": "حسب البلد",
    "by country-of-residence": "حسب بلد الإقامة",
    "by county": "حسب المقاطعة",
    "by culture": "حسب الثقافة",
    "by date": "حسب التاريخ",
    "by decade of establishment": "حسب عقد التأسيس",
    "by decade": "حسب العقد",
    "by dependent territory": "حسب الأقاليم التابعة",
    "by diocese": "حسب الأبرشية",
    "by director": "حسب المخرج",
    "by educational affiliation": "حسب الانتماء التعليمي",
    "by educational institution": "حسب الهيئة التعليمية",
    "by ethnic or national origin": "حسب الأصل العرقي أو الوطني",
    "by ethnicity": "حسب المجموعة العرقية",
    "by firing squad": "رميا بالرصاص",
    "by format": "حسب التنسيق",
    "by former country": "حسب البلد السابق",
    "by genre": "حسب النوع الفني",
    "by geographic setting": "حسب الموقع الجغرافي للأحداث",
    "by hamlet": "حسب القرية",
    "by hanging": "بالشنق",
    "by high school": "حسب المدرسة الثانوية",
    "by industry": "حسب الصناعة",
    "by instrument": "حسب الآلة",
    "by interest": "حسب الاهتمام",
    "by issue": "حسب القضية",
    "by language": "حسب اللغة",
    "by law enforcement officers": "بواسطة ضباط إنفاذ القانون",
    "by league": "حسب الدوري",
    "by line": "حسب الخط",
    "by location and occupation": "حسب الموقع والمهنة",
    "by location": "حسب الموقع",
    "by material": "حسب المادة",
    "by medium": "حسب الوسط",
    "by millennium": "حسب الألفية",
    "by mission country": "حسب بلد البعثة",
    "by month": "حسب الشهر",
    "by movement": "حسب الحركة",
    "by municipality": "حسب البلدية",
    "by nation and year": "حسب الموطن والسنة",
    "by national amateur team": "حسب المنتخب الوطني للهواة",
    "by national team": "حسب المنتخب الوطني",
    "by national women's team": "حسب منتخب السيدات الوطني",
    "by national youth team": "حسب المنتخب الوطني للشباب",
    "by nationality and century": "حسب الجنسية والقرن",
    "by nationality and competition": "حسب الجنسية والمنافسة",
    "by nationality and occupation": "حسب الجنسية والمهنة",
    "by nationality and status": "حسب الجنسية والحالة",
    "by nationality": "حسب الجنسية",
    "by newspaper": "حسب الصحيفة",
    "by occupation and century": "حسب المهنة والقرن",
    "by occupation and region": "حسب المهنة والمنطقة",
    "by occupation": "حسب المهنة",
    "by organization": "حسب المنظمة",
    "by parish": "حسب الأبرشية",
    "by party": "حسب الحزب",
    "by period by state": "حسب الحقبة حسب الولاية",
    "by period of introduction": "حسب حقبة الاستحداث",
    "by period": "حسب الحقبة",
    "by person": "حسب الشخص",
    "by populated place and occupation": "حسب المكان المأهول والمهنة",
    "by populated place": "حسب المكان المأهول",
    "by position": "حسب المركز",
    "by producer": "حسب المنتج",
    "by production location": "حسب موقع الإنتاج",
    "by province or territory": "حسب المقاطعة أو الإقليم",
    "by province": "حسب المقاطعة",
    "by publication": "حسب المؤسسة",
    "by region": "حسب المنطقة",
    "by religion": "حسب الدين",
    "by school": "حسب المدرسة",
    "by series": "حسب السلسلة",
    "by setting location": "حسب موقع الأحداث",
    "by shooting location": "حسب موقع التصوير",
    "by source": "حسب المصدر",
    "by specialty and nationality": "حسب التخصص والجنسية",
    "by sport and year": "حسب الرياضة والسنة",
    "by sport by state": "حسب الرياضة حسب الولاية",
    "by sport": "حسب الرياضة",
    "by stabbing": "بالطعن",
    "by state or region": "حسب الولاية أو المنطقة",
    "by state or territory": "حسب الولاية أو الإقليم",
    "by state or union territory": "حسب الولاية أو الإقليم الاتحادي",
    "by state": "حسب الولاية",
    "by status": "حسب الحالة",
    "by studio": "حسب استوديو الإنتاج",
    "by subfield": "حسب الحقل الفرعي",
    "by subgenre": "حسب النوع الفرعي",
    "by subject": "حسب الموضوع",
    "by team": "حسب الفريق",
    "by technology": "حسب التقانة",
    "by term": "حسب الفترة",
    "by time": "حسب الوقت",
    "by topic": "حسب الموضوع",
    "by town": "حسب البلدة",
    "by type": "حسب الفئة",
    "by under-20 national team": "حسب المنتخب الوطني تحت 20 سنة",
    "by under-21 national team": "حسب المنتخب الوطني تحت 21 سنة",
    "by under-23 national team": "حسب المنتخب الوطني تحت 23 سنة",
    "by university or college": "حسب الجامعة أو الكلية",
    "by village": "حسب القرية",
    "by violence": "بسبب العنف",
    "by war": "حسب الحرب",
    "by women's under-20 national team": "حسب المنتخب الوطني للسيدات تحت 20 سنة",
    "by women's under-21 national team": "حسب المنتخب الوطني للسيدات تحت 21 سنة",
    "by women's under-23 national team": "حسب المنتخب الوطني للسيدات تحت 23 سنة",
    "by writer": "حسب الكاتب",
    "by year and continent": "حسب السنة والقارة",
    "by year of disestablishment": "حسب سنة الانحلال",
    "by year of establishment": "حسب سنة التأسيس",
    "by year of formal description": "حسب سنة الوصف",
    "by year of introduction": "حسب سنة الاستحداث",
}

data2 = {
    "by century of closing": "حسب قرن الاغلاق",
    "by country of origin": "حسب البلد الأصل",
    "by organization": "حسب المنظمة",
    "by nonprofit organization": "حسب المنظمات غير الربحية",
    "by organization or nonprofit organization": "حسب المنظمة أو المنظمات غير الربحية",
    "by organization by nonprofit organization": "حسب المنظمة حسب المنظمات غير الربحية",
    "by organization and nonprofit organization": "حسب المنظمة والمنظمات غير الربحية",
}

to_test = [
    ("test_by_table_get", data1, resolve_by_labels),
    ("test_resolve_by_labels_1", data1, resolve_by_labels),
]


@pytest.mark.parametrize("category, expected", data1.items(), ids=data1.keys())
@pytest.mark.fast
def test_by_table_get(category: str, expected: str) -> None:
    label = resolve_by_labels(category)
    assert label == expected


@pytest.mark.parametrize("category, expected", data2.items(), ids=data2.keys())
@pytest.mark.fast
def test_by_table_get_2(category: str, expected: str) -> None:
    label2 = resolve_by_labels(category)
    assert label2 == expected


@pytest.mark.parametrize("name,data,callback", to_test)
@pytest.mark.dump
def test_dump_it(name: str, data: dict[str, str], callback) -> None:
    expected, diff_result = one_dump_test(data, callback)
    dump_diff(diff_result, name)
    assert diff_result == expected, f"Differences found: {len(diff_result):,}, len all :{len(data):,}"
